@@ -1,25 +1,22 @@
-package Tk::TList; 
+package Tk::TList;
 
-use vars qw($VERSION @ISA);
-$VERSION = '3.004'; # $Id: //depot/Tk8/TList/TList.pm#4$
+use vars qw($VERSION);
+$VERSION = '3.016'; # $Id: //depot/Tk8/TList/TList.pm#16 $
 
-use Tk qw(Ev);
+use Tk qw(Ev $XS_VERSION);
 
-@ISA = qw(Tk::Widget);
+use base  qw(Tk::Widget);
 
 use strict;
 
 Construct Tk::Widget 'TList';
 
-bootstrap Tk::TList $Tk::VERSION; 
+bootstrap Tk::TList;
 
 sub Tk_cmd { \&Tk::tlist }
 
-
-EnterMethods Tk::TList __FILE__,qw(insert anchor 
-                                   delete dragsite dropsite entrycget
-                                   entryconfigure info
-                                   nearest see selection xview yview);
+Tk::Methods qw(insert anchor delete dragsite dropsite entrycget
+               entryconfigure info nearest see selection xview yview);
 
 use Tk::Submethods ( 'delete' => [qw(all entry offsprings siblings)],
                      'info' => [qw(anchor dragsite dropsite selection)],
@@ -29,40 +26,24 @@ use Tk::Submethods ( 'delete' => [qw(all entry offsprings siblings)],
                      'dropsite' => [qw(clear set)],
                    );
 
-
 sub ClassInit
 {
  my ($class,$mw) = @_;
 
  $mw->bind($class,'<ButtonPress-1>',[ 'Button1' ] );
  $mw->bind($class,'<Shift-ButtonPress-1>',[ 'ShiftButton1' ] );
- $mw->bind($class,'<Control-ButtonRelease-1>', sub {} );
- $mw->bind($class,'<ButtonRelease-1>',
-		sub
-		 {
-		  my $w = shift;
-		  my $Ev = $w->XEvent;
-		  $w->CancelRepeat
-		      if($w->cget('-selectmode') ne "dragdrop");
-		  $w->ButtonRelease1($Ev);
-		 });
+ $mw->bind($class,'<Control-ButtonRelease-1>','Control_ButtonRelease_1');
+ $mw->bind($class,'<ButtonRelease-1>','ButtonRelease_1');
  $mw->bind($class,'<B1-Motion>',[ 'Button1Motion' ] );
  $mw->bind($class,'<B1-Leave>',[ 'AutoScan' ] );
 
  $mw->bind($class,'<Double-ButtonPress-1>',['Double1']);
 
- $mw->bind($class,'<Control-B1-Motion>', sub {} );
+ $mw->bind($class,'<Control-B1-Motion>','Control_B1_Motion');
  $mw->bind($class,'<Control-ButtonPress-1>',['CtrlButton1']);
  $mw->bind($class,'<Control-Double-ButtonPress-1>',['CtrlButton1']);
 
- $mw->bind($class,'<B1-Enter>',
-		sub
-		 {
-		  my $w = shift;
-		  my $Ev = $w->XEvent;
-		  $w->CancelRepeat
-		      if($w->cget('-selectmode') ne "dragdrop");
-		 });
+ $mw->bind($class,'<B1-Enter>','B1_Enter');
 
  $mw->bind($class,'<Up>',  ['DirKey', 'up']);
  $mw->bind($class,'<Down>',['DirKey', 'down']);
@@ -70,8 +51,8 @@ sub ClassInit
  $mw->bind($class,'<Left>', ['DirKey', 'left']);
  $mw->bind($class,'<Right>',['DirKey', 'right']);
 
- $mw->bind($class,'<Prior>', sub {shift->yview('scroll', -1, 'pages') } );
- $mw->bind($class,'<Next>',  sub {shift->yview('scroll',  1, 'pages') } );
+ $mw->bind($class,'<Prior>','Prior');
+ $mw->bind($class,'<Next>','Next');
 
  $mw->bind($class,'<Return>', ['KeyboardActivate']);
  $mw->bind($class,'<space>',  ['KeyboardBrowse']);
@@ -79,19 +60,60 @@ sub ClassInit
  return $class;
 }
 
+sub Control_ButtonRelease_1
+{
+}
+
+
+sub ButtonRelease_1
+{
+ my $w = shift;
+ my $Ev = $w->XEvent;
+ $w->CancelRepeat
+ if($w->cget('-selectmode') ne 'dragdrop');
+ $w->ButtonRelease1($Ev);
+}
+
+
+sub Control_B1_Motion
+{
+}
+
+
+sub B1_Enter
+{
+ my $w = shift;
+ my $Ev = $w->XEvent;
+ $w->CancelRepeat
+ if($w->cget('-selectmode') ne 'dragdrop');
+}
+
+
+sub Prior
+{
+shift->yview('scroll', -1, 'pages')
+}
+
+
+sub Next
+{
+shift->yview('scroll',  1, 'pages')
+}
+
+
 sub Button1
 {
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'}; 
+ delete $w->{'shiftanchor'};
 
  $w->focus()
-    if($w->cget("-takefocus"));
+    if($w->cget('-takefocus'));
 
- my $mode = $w->cget("-selectmode");
+ my $mode = $w->cget('-selectmode');
 
- if ($mode eq "dragdrop")
+ if ($mode eq 'dragdrop')
   {
    # $w->Send_WaitDrag($Ev->y);
    return;
@@ -99,36 +121,36 @@ sub Button1
 
  my $ent = $w->GetNearest($Ev->x, $Ev->y);
 
- return unless( $ent );
+ return unless defined $ent;
 
  my $browse = 0;
-   
- if($mode eq "single")
+
+ if($mode eq 'single')
   {
    $w->anchor('set', $ent);
   }
- elsif($mode eq "browse")
+ elsif($mode eq 'browse')
   {
    $w->anchor('set', $ent);
    $w->selection('clear' );
    $w->selection('set', $ent);
    $browse = 1;
   }
- elsif($mode eq "multiple")
+ elsif($mode eq 'multiple')
   {
    $w->selection('clear');
    $w->anchor('set', $ent);
    $w->selection('set', $ent);
    $browse = 1;
   }
- elsif($mode eq "extended")
+ elsif($mode eq 'extended')
   {
    $w->anchor('set', $ent);
    $w->selection('clear');
    $w->selection('set', $ent);
    $browse = 1;
   }
-   
+
  $w->Callback(-browsecmd => $ent) if ($browse);
 }
 
@@ -139,16 +161,16 @@ sub ShiftButton1
 
  my $to = $w->GetNearest($Ev->x,$Ev->y);
 
- delete $w->{'shiftanchor'}; 
+ delete $w->{'shiftanchor'};
 
- return unless($to);
+ return unless defined $to;
 
  my $mode = $w->cget('-selectmode');
 
- if($mode eq "extended")
+ if ($mode eq 'extended')
   {
    my $from = $w->info('anchor');
-   if($from)
+   if (defined $from)
     {
      $w->selection('clear');
      $w->selection('set', $from, $to);
@@ -178,11 +200,11 @@ sub ButtonRelease1
 {
  my ($w, $Ev) = @_;
 
- delete $w->{'shiftanchor'}; 
+ delete $w->{'shiftanchor'};
 
  my $mode = $w->cget('-selectmode');
 
- if($mode eq "dragdrop")
+ if($mode eq 'dragdrop')
   {
 #   $w->Send_DoneDrag();
    return;
@@ -191,29 +213,29 @@ sub ButtonRelease1
  my ($x, $y) = ($Ev->x, $Ev->y);
  my $ent = $w->GetNearest($x,$y);
 
- return unless($ent);
+ return unless defined $ent;
 
  if($x < 0 || $y < 0 || $x > $w->width || $y > $w->height)
   {
    $w->selection('clear');
 
-   return if($mode eq "single" || $mode eq "browse")
+   return if($mode eq 'single' || $mode eq 'browse')
 
   }
  else
   {
-   if($mode eq "single" || $mode eq "browse")
+   if($mode eq 'single' || $mode eq 'browse')
     {
      $w->anchor('set', $ent);
      $w->selection('clear');
      $w->selection('set', $ent);
 
     }
-   elsif($mode eq "multiple")
+   elsif($mode eq 'multiple')
     {
      $w->selection('set', $ent);
     }
-   elsif($mode eq "extended")
+   elsif($mode eq 'extended')
     {
      $w->selection('set', $ent);
     }
@@ -227,11 +249,11 @@ sub Button1Motion
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'}; 
+ delete $w->{'shiftanchor'};
 
  my $mode = $w->cget('-selectmode');
 
- if ($mode eq "dragdrop")
+ if ($mode eq 'dragdrop')
   {
 #   $w->Send_StartDrag();
    return;
@@ -239,16 +261,16 @@ sub Button1Motion
 
  my $ent = $w->GetNearest($Ev->x,$Ev->y);
 
- return unless($ent);
+ return unless defined $ent;
 
- if($mode eq "single")
+ if($mode eq 'single')
   {
    $w->anchor('set', $ent);
   }
- elsif($mode eq "multiple" || $mode eq "extended")
+ elsif($mode eq 'multiple' || $mode eq 'extended')
   {
    my $from = $w->info('anchor');
-   if($from)
+   if (defined $from)
     {
      $w->selection('clear');
      $w->selection('set', $from, $ent);
@@ -261,7 +283,7 @@ sub Button1Motion
     }
   }
 
- if($mode ne "single")
+ if($mode ne 'single')
   {
    $w->Callback(-browsecmd =>$ent);
   }
@@ -272,14 +294,13 @@ sub Double1
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'}; 
+ delete $w->{'shiftanchor'};
 
  my $ent = $w->GetNearest($Ev->x,$Ev->y);
 
- return unless($ent);
+ return unless defined $ent;
 
- $w->anchor('set', $ent)
-	unless($w->info('anchor'));
+ $w->anchor('set', $ent) unless defined($w->info('anchor'));
 
  $w->selection('set', $ent);
  $w->Callback(-command => $ent);
@@ -290,17 +311,17 @@ sub CtrlButton1
  my $w = shift;
  my $Ev = $w->XEvent;
 
- delete $w->{'shiftanchor'}; 
+ delete $w->{'shiftanchor'};
 
  my $ent = $w->GetNearest($Ev->x,$Ev->y);
 
- return unless( $ent );
+ return unless defined $ent;
 
  my $mode = $w->cget('-selectmode');
 
- if($mode eq "extended")
+ if($mode eq 'extended')
   {
-   $w->anchor('set', $ent) unless( $w->info('anchor') );
+   $w->anchor('set', $ent) unless defined( $w->info('anchor') );
 
    if($w->selection('includes', $ent))
     {
@@ -331,7 +352,7 @@ sub KeyboardActivate
 
  my $anchor = $w->info('anchor');
 
- return unless( $anchor );
+ return unless defined $anchor;
 
  if($w->cget('-selectmode'))
   {
@@ -347,7 +368,7 @@ sub KeyboardBrowse
 
  my $anchor = $w->info('anchor');
 
- return unless( $anchor );
+ return unless defined $anchor;
 
  if($w->cget('-selectmode'))
   {
@@ -361,8 +382,8 @@ sub AutoScan
 {
  my $w = shift;
 
- return if($w->cget('-selectmode') eq "dragdrop");
- 
+ return if($w->cget('-selectmode') eq 'dragdrop');
+
  my $Ev = $w->XEvent;
  my $y = $Ev->y;
  my $x = $Ev->x;
@@ -387,7 +408,7 @@ sub AutoScan
   {
    return;
   }
- $w->RepeatId($w->after(50,"AutoScan",$w));
+ $w->RepeatId($w->after(50,[AutoScan => $w]));
  $w->Button1Motion;
 }
 
