@@ -1,96 +1,126 @@
 #
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994 Sun Microsystems, Inc.
-# Copyright (c) 1995-1998 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 
-# modify it under the same terms as Perl itself, subject 
+# modify it under the same terms as Perl itself, subject
 # to additional disclaimer in Tk/license.terms due to partial
 # derivation from Tk8.0 sources.
 #
 package Tk;
-require 5.004;
+require 5.00404;
 use     AutoLoader qw(AUTOLOAD);
 use     DynaLoader;
-require Exporter;
-@Tk::ISA = qw(Exporter DynaLoader);
+use     Tk::Event ();
+use base qw(Exporter DynaLoader);
+
+*fileevent = \&Tk::Event::IO::fileevent;
 
 BEGIN { $Tk::platform = ($^O eq 'MSWin32') ? $^O : 'unix' };
 
-# XXX
-if ($Tk::platform eq 'unix') {
-    $Tk::tearoff = 1;
-}
+$Tk::tearoff = 1 if ($Tk::platform eq 'unix');
 
 @EXPORT    = qw(Exists Ev after exit MainLoop DoOneEvent tkinit);
-@EXPORT_OK = qw(NoOp *widget *event lsearch catch 
-                DONT_WAIT WINDOW_EVENTS  FILE_EVENTS TIMER_EVENTS 
-                IDLE_EVENTS ALL_EVENTS 
-                NORMAL_BG ACTIVE_BG SELECT_BG 
+@EXPORT_OK = qw(NoOp *widget *event lsearch catch $XS_VERSION
+                DONT_WAIT WINDOW_EVENTS  FILE_EVENTS TIMER_EVENTS
+                IDLE_EVENTS ALL_EVENTS
+                NORMAL_BG ACTIVE_BG SELECT_BG
                 SELECT_FG TROUGH INDICATOR DISABLED BLACK WHITE);
-%EXPORT_TAGS = (eventtypes => [qw(DONT_WAIT WINDOW_EVENTS  FILE_EVENTS 
-                                  TIMER_EVENTS IDLE_EVENTS ALL_EVENTS)], 
+%EXPORT_TAGS = (eventtypes => [qw(DONT_WAIT WINDOW_EVENTS  FILE_EVENTS
+                                  TIMER_EVENTS IDLE_EVENTS ALL_EVENTS)],
                 variables  => [qw(*widget *event)],
-                colors     => [qw(NORMAL_BG ACTIVE_BG SELECT_BG SELECT_FG 
+                colors     => [qw(NORMAL_BG ACTIVE_BG SELECT_BG SELECT_FG
                                   TROUGH INDICATOR DISABLED BLACK WHITE)],
                );
 
 use strict;
-use Symbol ();
 
 use Carp;
 
 # $tk_version and $tk_patchLevel are reset by pTk when a mainwindow
 # is created, $VERSION is checked by bootstrap
-$Tk::version     = "8.0";
-$Tk::patchLevel  = "8.0";
-$Tk::VERSION     = '800.0_02';
+$Tk::version     = '8.0';
+$Tk::patchLevel  = '8.0';
+$Tk::VERSION     = '800.016';
+$Tk::XS_VERSION  = $Tk::VERSION;
 $Tk::strictMotif = 0;
-                                   
+
 {($Tk::library) = __FILE__ =~ /^(.*)\.pm$/;}
 $Tk::library = Tk->findINC('.') unless (-d $Tk::library);
 
 $Tk::widget  = undef;
 $Tk::event   = undef;
 
-# Supress used once warnings on function table pointers 
-# How can we do this in the C code?
-use vars qw($TkVtab $TkintVtab $LangVtab $TkglueVtab $XlibVtab $TkoptionVtab);  
-use vars qw($TixVtab $TixintVtab $TiximgxpmVtab);
-use vars qw($TkwinVtab $TkwinintVtab);
- 
-bootstrap Tk $Tk::VERSION;
+use vars qw($inMainLoop);
+
+bootstrap Tk;
 
-{
- no strict 'refs';
- *{'exit'} = \&Exit;
-}
 my $boot_time = timeofday();
 
-# This is a workround for Solaris X11 locale handling 
-Preload(DynaLoader::dl_findfile('-L/usr/openwin/lib','-lX11')) if (&NeedPreload && -d '/usr/openwin/lib');
+# This is a workround for Solaris X11 locale handling
+Preload(DynaLoader::dl_findfile('-L/usr/openwin/lib','-lX11'))
+  if (NeedPreload() && -d '/usr/openwin/lib');
 
 use Tk::Submethods ('option'    =>  [qw(add get clear readfile)],
                     'clipboard' =>  [qw(clear append)]
                    );
 
+sub _backTrace
+{
+ my $w = shift;
+ my $i = 1;
+ my ($pack,$file,$line,$sub) = caller($i++);
+ while (1)
+  {
+   my $loc = "at $file line $line";
+   ($pack,$file,$line,$sub) = caller($i++);
+   last unless defined($sub);
+   return 1 if $sub eq '(eval)';
+   $w->AddErrorInfo("$sub $loc");
+  }
+ return 0;
+}
+
 sub BackTrace
 {
  my $w = shift;
  return unless (@_ || $@);
  my $mess = (@_) ? shift : "$@";
- my $i = 0;  
- my ($pack,$file,$line,$sub) = caller($i++);
- while (1)   
-  {          
-   my $loc = "at $file line $line";
-   ($pack,$file,$line,$sub) = caller($i++);
-   last if (!defined($sub) || $sub eq '(eval)');
-   $w->AddErrorInfo("$sub $loc");
-  }          
+ die "$mess\n" if $w->_backTrace;
+ # if we get here we are not in an eval so report now
+ $w->Fail($mess);
+ $w->idletasks;
  die "$mess\n";
 }
 
+#
+# This is a $SIG{__DIE__} handler which does not change the $@
+# string in the way 'croak' does, but rather add to Tk's ErrorInfo.
+# It stops at 1st enclosing eval on assumption that the eval
+# is part of Tk call process and will add its own context to ErrorInfo
+# and then pass on the error.
+#
+sub __DIE__
+{
+ my $mess = shift;
+ my $w = $Tk::widget;
+ # Note that if a __DIE__ handler returns it re-dies up the chain.
+ return unless defined $w;
+ return if $w->_backTrace;
+ # Not in an eval - should not happen
+}
+
+sub XEvent::xy { shift->Info('xy') }
+
+sub XEvent::AUTOLOAD
+{
+ my ($meth) = $XEvent::AUTOLOAD =~ /(\w)$/;
+ no strict 'refs';
+ *{$XEvent::AUTOLOAD} = sub { shift->Info($meth) };
+ goto &$XEvent::AUTOLOAD;
+}
+
 sub NoOp  { }
 
 sub Ev
@@ -102,11 +132,11 @@ sub Ev
    my $arg = pop(@args);
    $obj = (ref $arg) ? $arg : \$arg;
   }
- else 
+ else
   {
    $obj = \@args;
   }
- return bless $obj,"Tk::Ev";
+ return bless $obj,'Tk::Ev';
 }
 
 sub InitClass
@@ -134,7 +164,7 @@ sub Exists
 sub Time_So_Far
 {
  return timeofday() - $boot_time;
-} 
+}
 
 # Selection* are not autoloaded as names are too long.
 
@@ -145,17 +175,17 @@ sub SelectionOwn
 
 sub SelectionOwner
 {
- selection('own',"-displayof",@_);
+ selection('own','-displayof',@_);
 }
 
 sub SelectionClear
 {
- selection('clear',"-displayof",@_);
+ selection('clear','-displayof',@_);
 }
 
 sub SelectionExists
 {
- selection('exists',"-displayof",@_);
+ selection('exists','-displayof',@_);
 }
 
 sub SelectionHandle
@@ -164,61 +194,65 @@ sub SelectionHandle
  selection('handle',@_,$widget,$command);
 }
 
-#
-# This is a $SIG{__DIE__} handler which does not change the $@
-# string in the way 'croak' does, but rather add to Tk's ErrorInfo.
-# It stops at 1st enclosing eval on assumption that the eval
-# is part of Tk call process and will add its own context to ErrorInfo
-# and then pass on the error.
-# 
-sub __DIE__
-{
- my $mess = shift;
- my $w = $Tk::widget;
- if (defined $w)
-  {
-   my $i = 0;  
-   my ($pack,$file,$line,$sub) = caller($i++);
-   while (1)   
-    {          
-     my $loc = "at $file line $line";
-     ($pack,$file,$line,$sub) = caller($i++);
-     last if (!defined($sub) || $sub eq '(eval)');
-     $w->AddErrorInfo("$sub $loc");
-    }          
-  }
-}
-
 sub SplitString
 {
  local $_ = shift;
- # carp "SplitString '$_'";
  my (@arr, $tmp);
  while (/\{([^{}]*)\}|((?:[^\s\\]|\\.)+)/gs) {
    if (defined $1) { push @arr, $1 }
    else { $tmp = $2 ; $tmp =~ s/\\([\s\\])/$1/g; push @arr, $tmp }
- }  
+ }
  # carp '('.join(',',@arr).")";
  return @arr;
 }
 
 sub Methods
 {
- my ($package,$file) = caller;   
- $package->EnterMethods($file,@_);
+ my ($package) = caller;
+ no strict 'refs';
+ foreach my $meth (@_)
+  {
+   my $name = $meth;
+   *{$package."::$meth"} = sub { shift->WidgetMethod($name,@_) };
+  }
 }
 
-sub fileevent
-{
- require Tk::IO;
- goto &Tk::IO::fileevent;
-} 
+
+sub MessageBox {
+    my ($kind,%args) = @_;
+    require Tk::Dialog;
+    my $parent = delete $args{'-parent'};
+    my $args = \%args;
+
+    $args->{-bitmap} = delete $args->{-icon} if defined $args->{-icon};
+    $args->{-text} = delete $args->{-message} if defined $args->{-message};
+    $args->{-type} = 'OK' unless defined $args->{-type};
+
+    my $type;
+    if (defined($type = delete $args->{-type})) {
+	delete $args->{-type};
+	my @buttons = grep($_,map(ucfirst($_),
+                      split(/(abort|retry|ignore|yes|no|cancel|ok)/,
+                            lc($type))));
+	$args->{-buttons} = [@buttons];
+	$args->{-default_button} = delete $args->{-default} if
+	    defined $args->{-default};
+	if (not defined $args->{-default_button} and scalar(@buttons) == 1) {
+	   $args->{-default_button} = $buttons[0];
+	}  
+        my $md = $parent->Dialog(%$args);
+        my $an = $md->Show;
+        $md->destroy;
+        return $an;
+    }
+} # end messageBox
 
 sub messageBox
 {
- tk_messageBox(-parent => shift,@_);
+ my ($widget,%args) = @_;
+ $args{'-type'} = (exists $args{'-type'}) ? lc($args{'-type'}) : 'ok';
+ tk_messageBox(-parent => $widget, %args);
 }
-   
 
 sub getOpenFile
 {
@@ -233,11 +267,11 @@ sub getSaveFile
 sub chooseColor
 {
  tk_chooseColor(-parent => shift,@_);
-}               
+}
 
 sub DialogWrapper
 {
- my ($method,$kind,%args) = @_;      
+ my ($method,$kind,%args) = @_;
  my $created = 0;
  my $w = delete $args{'-parent'};
  if (defined $w)
@@ -246,10 +280,10 @@ sub DialogWrapper
   }
  else
   {
-   $w = MainWindow->new;                    
+   $w = MainWindow->new;
    $w->withdraw;
    $created = 1;
-  }                  
+  }
  my $mw = $w->MainWindow;
  my $fs = $mw->{$kind};
  unless (defined $fs)
@@ -272,42 +306,69 @@ sub ColorDialog
 }
 
 sub FDialog
-{           
- require Tk::FileSelect;  
- if ($_[0] =~ /Save/)
+{
+ require Tk::FBox;
+ my $cmd = shift;
+ if ($cmd =~ /Save/)
   {
-   push(@_,-create => 1, -verify => [qw(!-d -w)]);
+   push @_, -type => 'save';
   }
- DialogWrapper('FileSelect',@_);
+ DialogWrapper('FBox', $cmd, @_);
+}
+
+*MotifFDialog = \&FDialog;
+
+sub MainLoop
+{
+ unless ($inMainLoop)
+  {
+   local $inMainLoop = 1;
+   while (Tk::MainWindow->Count)
+    {
+     DoOneEvent(0);
+    }
+  }
+}
+
+sub tkinit { return MainWindow->new(@_) }
+
+# a wrapper on eval which turns off user $SIG{__DIE__}
+sub catch (&)
+{
+ my $sub = shift;
+ eval {local $SIG{'__DIE__'}; &$sub };
+}
+
+my $Home;
+
+sub TranslateFileName
+{
+ local $_ = shift;
+ unless (defined $Home)
+  {
+   $Home = $ENV{'HOME'} || ($ENV{'HOMEDRIVE'}.$ENV{'HOMEPATH'});
+   $Home =~ s#\\#/#g;
+   $Home .= '/' unless $Home =~ m#/$#;
+  }
+ s#~/#$Home#g;
+ # warn $_;
+ return $_;
 }
 
 1;
 
 __END__
-# provide an exit() to be exported if exit occurs 
-# before a MainWindow->new()
-sub exit { CORE::exit(@_);}
-
-sub Exists
-{my $w = shift;
- return defined($w) && ref($w) && $w->IsWidget && $w->exists;
-}
 
 sub Error
 {my $w = shift;
  my $error = shift;
  if (Exists($w))
   {
-   my $grab = $w->grab('current');  
+   my $grab = $w->grab('current');
    $grab->Unbusy if (defined $grab);
   }
  chomp($error);
- warn "Tk::Error: $error\n " . join("\n ",@_);
-}
-
-sub tkinit
-{
- return MainWindow->new(@_);
+ warn "Tk::Error: $error\n " . join("\n ",@_)."\n";
 }
 
 sub CancelRepeat
@@ -455,7 +516,7 @@ sub FocusOK
  if (!$@ && defined($value))
   {
    return 0 if ($value eq '0');
-   return 1 if ($value eq '1');
+   return $w->viewable if ($value eq '1');
    $value = $w->$value();
    return $value if (defined $value);
   }
@@ -464,7 +525,7 @@ sub FocusOK
    return 0;
   }
  catch { $value = $w->cget('-state') } ;
- if (!$@ && defined($value) && $value eq "disabled")
+ if (!$@ && defined($value) && $value eq 'disabled')
   {
    return 0;
   }
@@ -488,7 +549,7 @@ sub EnterFocus
  my $w  = shift;
  my $Ev = $w->XEvent;
  my $d  = $Ev->d;
- $w->Tk::focus() if ($d eq "NotifyAncestor" ||  $d eq "NotifyNonlinear" ||  $d eq "NotifyInferior");
+ $w->Tk::focus() if ($d eq 'NotifyAncestor' ||  $d eq 'NotifyNonlinear' ||  $d eq 'NotifyInferior');
 }
 
 sub tabFocus
@@ -499,7 +560,7 @@ sub tabFocus
 sub focusFollowsMouse
 {
  my $widget = shift;
- $widget->bind('all',"EnterFocus");
+ $widget->bind('all','<Enter>','EnterFocus');
 }
 
 # tkTraverseToMenu --
@@ -517,9 +578,8 @@ sub TraverseToMenu
 {
  my $w = shift;
  my $char = shift;
- return unless(defined $char && $char ne "");
+ return unless(defined $char && $char ne '');
  $w = $w->toplevel->FindMenu($char);
- $w->PostFirst() if (defined $w);
 }
 # tkFirstMenu --
 # This procedure traverses to the first menubutton in the toplevel
@@ -531,8 +591,7 @@ sub TraverseToMenu
 sub FirstMenu
 {
  my $w = shift;
- $w = $w->toplevel->FindMenu("");
- $w->PostFirst() if (defined $w);
+ $w = $w->toplevel->FindMenu('');
 }
 
 # These wrappers don't use method syntax so need to live
@@ -541,7 +600,7 @@ sub FirstMenu
 sub Selection
 {my $widget = shift;
  my $cmd    = shift;
- croak "Use SelectionOwn/SelectionOwner" if ($cmd eq 'own');
+ croak 'Use SelectionOwn/SelectionOwner' if ($cmd eq 'own');
  croak "Use Selection\u$cmd()";
 }
 
@@ -554,8 +613,8 @@ sub Clipboard
 sub Receive
 {
  my $w = shift;
- warn "Receive(" . join(',',@_) .")";
- die "Tk rejects send(" . join(',',@_) .")\n";
+ warn 'Receive(' . join(',',@_) .')';
+ die 'Tk rejects send(' . join(',',@_) .")\n";
 }
 
 sub break
@@ -617,10 +676,6 @@ sub lsearch
  return -1;
 }
 
-# a wrapper on eval which turns off user $SIG{__DIE__}
-sub catch (&)
-{
- my $sub = shift;
- eval {local $SIG{'__DIE__'}; &$sub };
-}
+
+
 
