@@ -1,24 +1,29 @@
 package Tk::ColorSelect;
-use strict;       
+use strict;
 
 use vars qw($VERSION);
-$VERSION = '3.007'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#7$
+$VERSION = '3.028'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#28 $
 
 use Tk qw(Ev);
 
 require Tk::Frame;
 
-use vars qw(@ISA); 
-@ISA = qw(Tk::Frame);
+use base  qw(Tk::Frame);
 Construct Tk::Widget 'ColorSelect';
 
 sub Populate
 {
-    my ($middle,$args) = @_; 
+    my ($middle,$args) = @_;
     my($i, @a);
-    foreach $i ('/usr/local/lib/X11/rgb.txt', '/usr/lib/X11/rgb.txt', 
+    require Tk::Config;
+    my(@xlibpath) = map { s/^-L//; "$_/X11/rgb.txt" }
+                    split /\s+/, $Tk::Config::xlib;
+    foreach $i (@xlibpath,
+		'/usr/local/lib/X11/rgb.txt', '/usr/lib/X11/rgb.txt',
+		'/usr/X11R6/lib/X11/rgb.txt',
                 '/usr/local/X11R5/lib/X11/rgb.txt', '/X11/R5/lib/X11/rgb.txt',
                 '/X11/R4/lib/rgb/rgb.txt', '/usr/openwin/lib/X11/rgb.txt') {
+        local *FOO;
         next if ! open FOO, $i;
         my $middle_left = $middle->Frame;
         $middle_left->pack(
@@ -38,44 +43,51 @@ sub Populate
 
         my $scroll = $middle->Scrollbar(
             -orient      => 'vertical',
-            -command     => ["yview", $names],
+            -command     => ['yview', $names],
             -relief      => 'sunken',
             -borderwidth => 2,
         );
-        $names->configure(-yscrollcommand => ["set",$scroll]);
+        $names->configure(-yscrollcommand => ['set',$scroll]);
         $names->pack(-in => $middle_left, -side => 'left');
         $scroll->pack(-in => $middle_left, -side => 'right', -fill => 'y');
 
-
         while(<FOO>) {
             chomp;
-            my @a = split /\s+/;
-            if (@a == 4)
-             {
-              my $hex = $middle->Hex($a[3]);
-              if (!exists($Tk::ColorEditor::names{$hex}) || 
-                  length($Tk::ColorEditor::names{$hex}) > length($a[3]))
-               {
-                $Tk::ColorEditor::names{$hex} = $a[3];
-                $names->insert('end', $a[3]);
-               }
-             }
-          }
+            next if /^!/;
+            my @a = split;
+            my $color = join(' ', @a[3 .. $#a]);
+            my $hex;
+	    eval { $hex = $middle->Hex($color); };
+            if ($@) {
+		#print STDERR "unknown color: '$color'\n";
+	        if ($@ =~ /unknown color name "/) {
+		    next;
+		} else {
+		    chomp $@;
+		    die $@;
+		}
+            }
+            if (!exists($Tk::ColorEditor::names{$hex}) ||
+                length($Tk::ColorEditor::names{$hex}) > length($color)) {
+                  $Tk::ColorEditor::names{$hex} = $color;
+                $names->insert('end', $color);
+            }
+        }
         close FOO;
         last;
     }
 
-    # Create the three scales for editing the color, and the entry for typing 
+    # Create the three scales for editing the color, and the entry for typing
     # in a color value.
 
     my $middle_middle = $middle->Frame;
     $middle_middle->pack(-side => 'left', -expand => 1, -fill => 'y');
     my $mcm1 = $middle_middle->Optionmenu(-variable => \$middle->{'color_space'},
-                                  -command => [ $middle, 'color_space'], 
+                                  -command => [ $middle, 'color_space'],
                                   -relief  => 'raised',
-                                  -options => [ ['RGB color space' => 'rgb'],
-                                                ['CMY color space' => 'cmy'],
-                                                ["HSB color space" => 'hsb']]);
+                                  -options => [ ['HSB color space' => 'hsb'],
+                                                ['RGB color space' => 'rgb'],
+                                                ['CMY color space' => 'cmy']]);
     $mcm1->pack(-side => 'top', -fill => 'x');
 
     my(@middle_middle, @label, @scale);
@@ -87,7 +99,7 @@ sub Populate
     $middle_middle[1]->pack(-side => 'top', -expand => 1);
     $middle_middle[2]->pack(-side => 'top', -expand => 1);
     $middle_middle[3]->pack(-side => 'top', -expand => 1, -fill => 'x');
-    $middle->{'Labels'} = ["zero","one","two"];
+    $middle->{'Labels'} = ['zero','one','two'];
     foreach $i (0..2) {
         $label[$i] = $middle->Label(-textvariable => \$middle->{'Labels'}[$i]);
         $scale[$i] = $middle->Scale(
@@ -99,7 +111,7 @@ sub Populate
         );
         $scale[$i]->pack(
             -in     => $middle_middle[$i],
-            -side   => 'top', 
+            -side   => 'top',
             -anchor => 'w',
         );
         $label[$i]->pack(
@@ -108,20 +120,22 @@ sub Populate
             -anchor => 'w',
         );
     }
-    my $nameLabel = $middle->Label(-text => "Name:");
+    my $nameLabel = $middle->Label(-text => 'Name:');
+    $middle->{'Entry'} = '';
     my $name = $middle->Entry(
         -relief       => 'sunken',
         -borderwidth  => 2,
         -textvariable => \$middle->{'Entry'},
-        -width        => 10,
-        -font         => "-*-Courier-Medium-R-Normal--*-120-*-*-*-*-*-*"
-    );                   
+        -width        => 10,     
+# For some reason giving this font causes problems at end of t/create.t
+#       -font         => '-*-Courier-Medium-R-Normal--*-120-*-*-*-*-*-*'  
+    );
 
     $nameLabel->pack(-in => $middle_middle[3], -side => 'left');
     $name->pack(
         -in     => $middle_middle[3],
-        -side   => 'right', 
-        -expand => 1, 
+        -side   => 'right',
+        -expand => 1,
         -fill   => 'x',
     );
     $name->bind('<Return>' => [ $middle, 'color', Ev(['get'])]);
@@ -142,29 +156,28 @@ sub Populate
     my $swatch_item = $swatch->create('oval', '.5c', '.3c', '2.26c', '4.76c');
 
     my $value = $middle->Label(
-        -textvariable => \$middle->{'color'}, 
+        -textvariable => \$middle->{'color'},
         -width        => 13,
-        -font         => "-*-Courier-Medium-R-Normal--*-120-*-*-*-*-*-*"
-    );                   
+        -font         => '-*-Courier-Medium-R-Normal--*-120-*-*-*-*-*-*'
+    );
 
     $swatch->pack(
         -in     => $middle_right,
-        -side   => 'top', 
-        -expand => 1, 
+        -side   => 'top',
+        -expand => 1,
         -fill   => 'both',
     );
-    $value->pack(-in => $middle_right, -side => 'bottom', -pady => '.25c');   
+    $value->pack(-in => $middle_right, -side => 'bottom', -pady => '.25c');
 
     $middle->ConfigSpecs(
         '-color_space'  => ['METHOD', undef, undef, 'hsb'],
         '-initialcolor' => '-color',
-        '-color'        => ['METHOD', 'background', 'Background', 
+        '-color'        => ['METHOD', 'background', 'Background',
                                $middle->cget('-background')]
     );
 
     $middle->{'swatch'} = $swatch;
     $middle->{'swatch_item'} = $swatch_item;
-    $middle->{'Entry'} = '';
     $middle->{'scale'} = [@scale];
     $middle->{'red'} = 0;
     $middle->{'blue'} = 0;
@@ -176,7 +189,7 @@ sub Hex
 {
  my $w = shift;
  my @rgb = (@_ == 3) ? @_ : $w->rgb(@_);
- sprintf("#%04x%04x%04x",@rgb)
+ sprintf('#%04x%04x%04x',@rgb)
 }
 
 sub color_space {
@@ -188,21 +201,21 @@ sub color_space {
       my %Labels = ( 'rgb' => [qw(Red Green Blue)],
                      'cmy' => [qw(Cyan Magenta Yellow)],
                      'hsb' => [qw(Hue Saturation Brightness)] );
-                          
+
       # The procedure below is invoked when a new color space is selected. It
-      # changes the labels on the scales and re-loads the scales with the 
+      # changes the labels on the scales and re-loads the scales with the
       # appropriate values for the current color in the new color space
-                          
+
       $space = 'hsb' unless (exists $Labels{$space});
       my $i;
-      for $i (0..2)       
-       {                  
+      for $i (0..2)
+       {
         $objref->{'Labels'}[$i] = $Labels{$space}->[$i];
-       }                  
+       }
       $objref->{'color_space'} = $space;
       $objref->DoWhenIdle(['set_scales',$objref]) unless ($objref->{'pending'}++);
      }
- return $objref->{'color_space'}; 
+ return $objref->{'color_space'};
 } # color_space
 
 sub hsvToRgb {
@@ -239,22 +252,25 @@ sub color
  my ($objref,$name) = @_;
  if (@_ > 1 && defined($name) && length($name))
   {
-   # $objref->{'color'} = $name;
+      if ($name eq 'cancel') {
+	  $objref->{color} = undef;
+	  return;
+      }
    my ($format, $shift);
    my ($red, $green, $blue);
 
-   if ($name !~ /^#/) 
+   if ($name !~ /^#/)
     {
      ($red, $green, $blue) = $objref->{'swatch'}->rgb($name);
-    } 
-   else 
+    }
+   else
     {
        my $len = length $name;
-       if($len == 4) { $format = "#(.)(.)(.)"; $shift = 12; }
-         elsif($len == 7) { $format = "#(..)(..)(..)"; $shift = 8; }
-           elsif($len == 10) { $format = "#(...)(...)(...)"; $shift = 4; }
-             elsif($len == 13) { $format = "#(....)(....)(....)"; $shift = 0; }
-       else { 
+       if($len == 4) { $format = '#(.)(.)(.)'; $shift = 12; }
+         elsif($len == 7) { $format = '#(..)(..)(..)'; $shift = 8; }
+           elsif($len == 10) { $format = '#(...)(...)(...)'; $shift = 4; }
+             elsif($len == 13) { $format = '#(....)(....)(....)'; $shift = 0; }
+       else {
 	 $objref->BackTrace(
 	   "ColorEditor error:  syntax error in color name \"$name\"");
 	 return;
@@ -269,7 +285,7 @@ sub color
    $objref->{'red'} = $red;
    $objref->{'blue'} = $blue;
    $objref->{'green'} = $green;
-   my $hex = sprintf("#%04x%04x%04x", $red, $green, $blue);
+   my $hex = sprintf('#%04x%04x%04x', $red, $green, $blue);
    $objref->{'color'} = $hex;
    $objref->{'Entry'} = $name;
    $objref->DoWhenIdle(['set_scales',$objref]) unless ($objref->{'pending'}++);
@@ -279,8 +295,6 @@ sub color
  return $objref->{'color'};
 }
 
-
-
 sub rgbToHsv {
 
     # The procedure below converts an RGB value to HSB.  It takes red, green,
@@ -316,11 +330,10 @@ sub rgbToHsv {
 
 } # end rgbToHsv
 
-
 sub scale_changed {
 
     # The procedure below is invoked when one of the scales is adjusted.  It
-    # propagates color information from the current scale readings to 
+    # propagates color information from the current scale readings to
     # everywhere else that it is used.
 
     my($objref) = @_;
@@ -344,7 +357,7 @@ sub scale_changed {
     $objref->{'red'} = $red;
     $objref->{'blue'} = $blue;
     $objref->{'green'} = $green;
-    $objref->color(sprintf("#%04x%04x%04x", $red, $green, $blue));
+    $objref->color(sprintf('#%04x%04x%04x', $red, $green, $blue));
     $objref->idletasks;
 
 } # end scale_changed
@@ -380,11 +393,9 @@ sub set_scales {
 
 } # end set_scales
 
-
 package Tk::ColorDialog;
 require Tk::Toplevel;
-use vars qw(@ISA);
-@ISA = qw(Tk::Toplevel); 
+use base  qw(Tk::Toplevel);
 
 Construct Tk::Widget 'ColorDialog';
 
@@ -398,15 +409,16 @@ sub Accept
 sub Cancel
 {
  my $cw  = shift;
- $cw->configure(-color => undef);
+# $cw->configure(-color => undef);
+ $cw->configure(-color => 'cancel');
  $cw->Accept;
 }
 
 sub Populate
 {
- my ($cw,$args) = @_;       
+ my ($cw,$args) = @_;
  $cw->SUPER::Populate($args);
- $cw->protocol('WM_DELETE_WINDOW' => [ Cancel => $cw ]);
+ $cw->protocol('WM_DELETE_WINDOW' => [ 'Cancel' => $cw ]);
  $cw->transient($cw->Parent->toplevel);
  $cw->withdraw;
  my $sel = $cw->ColorSelect;
@@ -418,11 +430,10 @@ sub Populate
 }
 
 sub Show
-{           
+{
  my $cw = shift;
  $cw->configure(@_) if @_;
- $cw->Popup(); 
- $cw->waitVisibility;
+ $cw->Popup();
  $cw->waitVariable(\$cw->{'done'});
  $cw->withdraw;
  return $cw->cget('-color');
@@ -430,18 +441,17 @@ sub Show
 
 package Tk::ColorEditor;
 
-
-use vars qw($VERSION $SET_PALETTE @ISA);
-$VERSION = '3.007'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#7$
+use vars qw($VERSION $SET_PALETTE);
+$VERSION = '3.028'; # $Id: //depot/Tk8/Tk/ColorEditor.pm#28 $
 
 use Tk qw(lsearch Ev);
 use Tk::Toplevel;
-@ISA = qw(Tk::Toplevel);
+use base  qw(Tk::Toplevel);
 Construct Tk::Widget 'ColorEditor';
 
 %Tk::ColorEditor::names = ();
 
- 
+
 use Tk::Dialog;
 use Tk::Pretty;
 
@@ -457,11 +467,11 @@ sub add_menu_item
  my $value;
  foreach $value (@_)
   {
-   if ($value eq 'SEP') 
+   if ($value eq 'SEP')
     {
      $objref->{'mcm2'}->separator;
-    } 
-   else 
+    }
+   else
     {
      $objref->{'mcm2'}->command( -label => $value,
            -command => [ 'configure', $objref, '-highlight' => $value ] );
@@ -500,14 +510,13 @@ sub title
  return $w->{Configure}{'-title'};
 }
 
-
 sub delete_menu_item
 {
  my $objref = shift;
  my $value;
  foreach $value (@_)
   {
-   $objref->{'mcm2'}->delete($value);                                      
+   $objref->{'mcm2'}->delete($value);
    my $list_ord = $value =~ /\d+/ ? $value : lsearch($objref->{'highlight_list'}, $value);
    splice(@{$objref->{'highlight_list'}}, $list_ord, 1) if $list_ord != -1;
   }
@@ -522,7 +531,7 @@ sub configure {
     my($option, $value);
     while (($option, $value) = splice(@hook_list, 0, 2)) {
 	$objref->SUPER::configure($option => $value);
-    } # whilend all options/values          
+    } # whilend all options/values
 
 } # end configure
 
@@ -549,12 +558,12 @@ sub delete_widgets {
 
 } # end delete_widgets
 
-sub ApplyDefault 
+sub ApplyDefault
 {
  my($objref) = @_;
  my $cb = $objref->cget('-command');
  my $h;
- foreach $h (@{$objref->{'highlight_list'}}) 
+ foreach $h (@{$objref->{'highlight_list'}})
   {
    next if $h =~ /TEAR_SEP|SEP/;
    $cb->Call($h);
@@ -578,7 +587,7 @@ sub Populate
         foreground background SEP
         activeForeground activeBackground SEP
         highlightColor highlightBackground SEP
-        selectForeground selectBackground SEP 
+        selectForeground selectBackground SEP
         disabledForeground insertBackground selectColor troughColor
     );
 
@@ -586,11 +595,11 @@ sub Populate
 
     my $usage = $cw->Dialog( '-title' => 'ColorEditor Usage',
         -justify    => 'left',
-        -wraplength => '6i',				   
+        -wraplength => '6i',
         -text       => "The Colors menu allows you to:\n\nSelect a color attribute such as \"background\" that you wish to colorize.  Click on \"Apply\" to update that single color attribute.\n\nSelect one of three color spaces.  All color spaces display a color value as a hexadecimal number under the oval color swatch that can be directly supplied on widget commands.\n\nApply Tk's default color scheme to the application.  Useful if you've made a mess of things and want to start over!\n\nChange the application's color palette.  Make sure \"background\" is selected as the color attribute, find a pleasing background color to apply to all current and future application widgets, then select \"Set Palette\".",
     );
 
-    # Create the menu bar at the top of the window for the File, Colors 
+    # Create the menu bar at the top of the window for the File, Colors
     # and Help menubuttons.
 
     my $m0 = $cw->Frame(-relief => 'raised', -borderwidth => 2);
@@ -604,8 +613,8 @@ sub Populate
     $mf->pack(-side => 'left');
     my $close_command = [sub {shift->withdraw}, $cw];
     $mf->command(
-        -label       => 'Close', 
-        -underline   => 0, 
+        -label       => 'Close',
+        -underline   => 0,
         -command     => $close_command,
         -accelerator => 'Ctrl-w',
     );
@@ -624,15 +633,15 @@ sub Populate
     $mc->separator;
 
     $mc->command(
-        -label     => 'Apply Default Colors', 
+        -label     => 'Apply Default Colors',
         -underline => 6,
         -command   => ['ApplyDefault',$cw]
     );
     $mc->separator;
     $mc->command(
-        -label     => $SET_PALETTE, 
+        -label     => $SET_PALETTE,
         -underline => 0,
-        -command   => sub { $cw->setPalette($cw->cget('-color'))} 
+        -command   => sub { $cw->setPalette($cw->cget('-color'))}
     );
 
     my $m1 = $mc->cget(-menu);
@@ -647,8 +656,8 @@ sub Populate
     );
     $mh->pack(-side => 'right');
     $mh->command(
-        -label       => 'Usage', 
-        -underline   => 0, 
+        -label       => 'Usage',
+        -underline   => 0,
         -command     => [sub {shift->Show}, $usage],
     );
 
@@ -666,7 +675,7 @@ sub Populate
     );
     $update->pack(-pady => 1, -padx => '0.25c');
 
-    # Create the listbox that holds all of the color names in rgb.txt, if an 
+    # Create the listbox that holds all of the color names in rgb.txt, if an
     # rgb.txt file can be found.
 
     my $middle = $cw->ColorSelect(-relief => 'raised', -borderwidth => 2);
@@ -682,7 +691,7 @@ sub Populate
     $cw->{'highlight_list'} = [@highlight_list];
     $cw->{'mcm2'} = $mcm2;
 
-    foreach (@highlight_list) 
+    foreach (@highlight_list)
      {
       next if /^TEAR_SEP$/;
       $cw->add_menu_item($_);
@@ -696,12 +705,12 @@ sub Populate
     $cw->{'gwt_depth'} = 0;
     $cw->{'palette'} = $mc;
 
-    my $pixmap = $cw->Pixmap('-file' => Tk->findINC("ColorEdit.xpm"));
+    my $pixmap = $cw->Pixmap('-file' => Tk->findINC('ColorEdit.xpm'));
     $cw->Icon(-image => $pixmap);
 
-    $cw->ConfigSpecs(   
+    $cw->ConfigSpecs(
         DEFAULT         => [$middle],
-        -widgets        => ['PASSIVE', undef, undef, 
+        -widgets        => ['PASSIVE', undef, undef,
                                [$cw->parent->Descendants]],
         -display_status => ['PASSIVE', undef, undef, 0],
         '-title'        => ['METHOD', undef, undef, ''],
@@ -740,10 +749,10 @@ sub set_colors {
     foreach $widget (@{$objref->cget('-widgets')}) {
         if ($display) {
             $objref->{'Status_l'}->configure(
-                -text => "WIDGET:  " . $widget->PathName
+                -text => 'WIDGET:  ' . $widget->PathName
             );
             $objref->update;
-        } 
+        }
         eval {local $SIG{'__DIE__'}; $color = ($widget->configure("-\L${type}"))[3]} if $reset;
         eval {local $SIG{'__DIE__'}; $widget->configure("-\L${type}" => $color)};
     }
@@ -756,122 +765,7 @@ sub set_colors {
 
 1;
 
-__END__ 
-
-=head1 NAME 
-
-ColorEditor - a general purpose Tk widget Color Editor (based on tcolor.tcl
-from the Tcl/Tk distribution).
-
-=head1 SYNOPSIS
-
-   use Tk::ColorEditor;
-
-   $cref = $mw->ColorEditor(-title => $title, -cursor => @cursor);
-
-   $cref->Show;
-
-=head1 DESCRIPTION
-
-ColorEditor is implemented as an object with various methods, described 
-below.  First, create your ColorEditor object during program initialization 
-(one should be sufficient), and then configure it by specifying a list of Tk
-widgets to colorize. When it's time to use the editor, invoke the Show()
-method.
-
-ColorEditor allows some customization: you may alter the color attribute
-menu by adding and/or deleting menu items and/or separators, turn the status
-window on or off, alter the configurator's list of color widgets, or even 
-supply your own custom color configurator callback.
-
-=over 4
-
-=item 1.
-
-Call the constructor to create the editor object, which in turn returns a
-blessed reference to the new object:
-
-   use Tk::ColorEditor;
-
-   $cref = $mw->ColorEditor(
-       -title  => $title,
-       -cursor => @cursor,
-   );
-
-      mw     - a window reference, usually the result of a MainWindow->new
-               call.  As the default root of a widget tree, $mw and all
-               descendant widgets at object-creation-time are configured
-               by the default color configurator procedure.  (You probably
-               want to change this though or you might end up colorizing
-               ColorEditor!)
-      title  - Toplevel title, default = ' '.
-      cursor - a valid Tk '-cursor' specification (default is 
-               'top_left_arrow').  This cursor is used over all ColorEditor
-               "hot spots".
-
-=item 2.
-
-Invoke the configure() method to change editor characteristics:
-
-   $cref->configure(-option => value, ..., -option-n => value-n);
-   
-      options:
-        -command             : a callback to a  `set_colors' replacement.
-        -widgets             : a reference to a list of widget references
-                               for the color configurator.
-        -display_status      : TRUE IFF display the ColorEditor status
-                               window when applying colors.
-        -add_menu_item       : 'SEP', or a color attribute menu item.
-        -delete_menu_item    : 'SEP', a color attribute menu item, or color
-                               attribute menu ordinal.
-
-   For example:
-
-      $cref->configure(-delete_menu_item   => 3,
-          -delete_menu_item   => 'disabledforeground',
-          -add_menu_item      => 'SEP',
-          -add_menu_item      => 'New color attribute',
-          -widgets            => [$ce, $qu, $f2b2],
-          -widgets            => [$f2->Descendants],
-          -command            => [\&my_special_configurator, some, args ]
-      );
-
-=item 3.
-
-Invoke the Show() method on the editor object, say, by a button or menu press:
-
-   $cref->Show;
-
-=item 4.
-
-The cget(-widgets) method returns a reference to a list of widgets that
-are colorized by the configurator.  Typically, you add new widgets to
-this list and then use it in a subsequent configure() call to expand your
-color list.
-
-   $cref->configure(
-       -widgets => [
-           @{$Filesystem_ref->cget(-widgets)}, @{$cref->cget(-widgets)},
-       ]
-   );
-
-=item 5.
-
-The delete_widgets() method expects a reference to a list of widgets which are
-then removed from the current color list.
-
-   $cref->delete_widgets($OBJTABLE{$objname}->{'-widgets'})
-
-=back 
-
-=head1 AUTHORS
-
-Stephen O. Lidie, Lehigh University Computing Center.  95/03/05
-lusol@Lehigh.EDU
-
-Many thanks to Guy Decoux (decoux@moulon.inra.fr) for doing the initial 
-translation of tcolor.tcl to TkPerl, from which this code has been derived.
-
-=cut 
+__END__
 
+=cut
 
