@@ -1,4 +1,4 @@
-# Copyright (c) 1995-1998 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Derived;
@@ -8,56 +8,15 @@ use strict;
 use Carp;
 
 use vars qw($VERSION);
-$VERSION = '3.007'; # $Id: //depot/Tk8/Tk/Derived.pm#7$
-
-
-
-=head1 NAME
-
-Tk::Derived - Base class for widgets derived from others
-
-=head1 SYNOPSIS
-
-  package Tk::Whatever;
-  require Tk::Something; 
-  require Tk::Derived;
-
-  @ISA = qw(Tk::Derived Tk::Something);
-
-  sub Populate
-  {
-   my ($cw,$args) = @_;
-   ...
-   $cw->SUPER::Populate($args);
-   $cw->ConfigSpecs(...);
-   ...
-  }
-
-=head1 DESCRIPTION 
-
-Tk::Derived is used with perl5's multiple inheritance to override some
-methods normally inherited from Tk::Widget.
-
-Tk::Derived should precede any Tk widgets in the class's @ISA.
-
-Tk::Derived's main purpose is to apply wrappers to C<configure> and C<cget>
-methods of widgets to allow the derived widget to add to or modify behaviour
-of the configure options supported by the base widget.
-
-The derived class should normally override the C<Populate> method provided
-by Tk::Derived and call C<ConfigSpecs> to declare configure options.
-
-The public methods provided by Tk::Derived are as follows:
-
-=over 4
-
-=cut
+$VERSION = '3.029'; # $Id: //depot/Tk8/Tk/Derived.pm#29 $
 
 $Tk::Derived::Debug = 0;
 
+my $ENHANCED_CONFIGSPECS = 0; # disable for now
+
 use Tk qw(NORMAL_BG BLACK);
 
-sub Subwidget 
+sub Subwidget
 {
  my $cw = shift;
  my @result = ();
@@ -77,18 +36,18 @@ sub Subwidget
     }
   }
  return (wantarray) ? @result : $result[0];
-} 
+}
 
 sub _makelist
 {
  my $widget = shift;
- my (@specs) = (ref $widget && ref $widget eq "ARRAY") ? (@$widget) : ($widget);
+ my (@specs) = (ref $widget && ref $widget eq 'ARRAY') ? (@$widget) : ($widget);
  return @specs;
 }
 
 sub Subconfigure
 {
- # This finds the widget or widgets to to which to apply a particular 
+ # This finds the widget or widgets to to which to apply a particular
  # configure option
  my ($cw,$opt) = @_;
  my $config = $cw->ConfigSpecs;
@@ -113,7 +72,7 @@ sub Subconfigure
  $widget = $config->{DEFAULT} unless (defined $widget);
  if (defined $widget)
   {
-   $cw->BackTrace("Invalid ConfigSpecs $widget") unless (ref($widget) && (ref $widget eq "ARRAY"));
+   $cw->BackTrace("Invalid ConfigSpecs $widget") unless (ref($widget) && (ref $widget eq 'ARRAY'));
    $widget = $widget->[0];
   }
  else
@@ -122,12 +81,13 @@ sub Subconfigure
   }
  foreach $widget (_makelist($widget))
   {
+   $widget = 'SELF' if (ref($widget) && $widget == $cw);
    if (ref $widget)
     {
      my $ref = ref $widget;
      if ($ref eq 'ARRAY')
       {
-       $widget = Tk::Configure->new(@$widget); 
+       $widget = Tk::Configure->new(@$widget);
        push(@subwidget,$widget)
       }
      elsif ($ref eq 'HASH')
@@ -135,7 +95,7 @@ sub Subconfigure
        my $key;
        foreach $key (%$widget)
         {
-         my $sw;  
+         my $sw;
          foreach $sw (_makelist($widget->{$key}))
           {
            push(@subwidget,Tk::Configure->new($sw,$key));
@@ -153,11 +113,11 @@ sub Subconfigure
     }
    elsif ($widget eq 'DESCENDANTS')
     {
-     push(@subwidget,$cw->Descendants) 
+     push(@subwidget,$cw->Descendants)
     }
    elsif ($widget eq 'CHILDREN')
     {
-     push(@subwidget,$cw->children) 
+     push(@subwidget,$cw->children)
     }
    elsif ($widget eq 'METHOD')
     {
@@ -171,13 +131,13 @@ sub Subconfigure
     }
    elsif ($widget eq 'SELF')
     {
-     push(@subwidget,Tk::Configure->new('configure_self', 'cget_self', $cw,@arg))
+     push(@subwidget,Tk::Configure->new('Tk::configure', 'Tk::cget', $cw,@arg))
     }
-   elsif ($widget eq 'PASSIVE') 
+   elsif ($widget eq 'PASSIVE')
     {
      push(@subwidget,Tk::Configure->new('_configure','_cget',$cw,@arg))
     }
-   elsif ($widget eq 'CALLBACK') 
+   elsif ($widget eq 'CALLBACK')
     {
      push(@subwidget,Tk::Configure->new('_callback','_cget',$cw,@arg))
     }
@@ -193,21 +153,21 @@ sub Subconfigure
 sub _cget
 {
  my ($cw,$opt) = @_;
- $cw->BackTrace("Wrong number of args to cget") unless (@_ == 2);
+ $cw->BackTrace('Wrong number of args to cget') unless (@_ == 2);
  return $cw->{Configure}{$opt}
 }
 
 sub _configure
 {
  my ($cw,$opt,$val) = @_;
- $cw->BackTrace("Wrong number of args to configure") unless (@_ == 3);
+ $cw->BackTrace('Wrong number of args to configure') unless (@_ == 3);
  $cw->{Configure}{$opt} = $val;
 }
 
 sub _callback
 {
  my ($cw,$opt,$val) = @_;
- $cw->BackTrace("Wrong number of args to configure") unless (@_ == 3);
+ $cw->BackTrace('Wrong number of args to configure') unless (@_ == 3);
  $val = Tk::Callback->new($val) if defined $val;
  $cw->{Configure}{$opt} = $val;
 }
@@ -239,13 +199,13 @@ sub configure
 {
  # The default composite widget configuration method uses hash stored
  # in the widget's hash to map configuration options
- # onto subwidgets. 
- #                
+ # onto subwidgets.
+ #
  my @results = ();
  my $cw = shift;
  if (@_ <= 1)
   {
-   # Enquiry cases 
+   # Enquiry cases
    my $spec = $cw->ConfigSpecs;
    if (@_)
     {
@@ -260,9 +220,11 @@ sub configure
       {
        if (ref $info)
         {
-         unless (defined $info->[3])
+         # If the default slot is undef then ask subwidgets in turn
+         # for their default value until one accepts it.
+         if ($ENHANCED_CONFIGSPECS && !defined($info->[3]))
           {local $SIG{'__DIE__'};
-           my @def;              
+           my @def;
            foreach my $sw ($cw->Subconfigure($opt))
             {
              eval { @def = $sw->configure($opt) };
@@ -276,7 +238,7 @@ sub configure
         }
        else
         {
-         # Real (core) Tk widgets return db name rather than option name 
+         # Real (core) Tk widgets return db name rather than option name
          # for aliases so recurse to get that ...
          my @real = $cw->configure($info);
          push(@results,$opt,$real[1]);
@@ -327,9 +289,9 @@ sub configure
        next unless (defined $subwidget);
        eval {local $SIG{'__DIE__'};  $subwidget->configure($opt => $val) };
        if ($@)
-        {   
+        {
          my $val2 = (defined $val) ? $val : 'undef';
-         $error = "Can't set $opt to `$val2' for $cw: " . $@; 
+         $error = "Can't set $opt to `$val2' for $cw: " . $@;
          undef $@;
         }
        else
@@ -339,7 +301,7 @@ sub configure
       }
      $cw->BackTrace($error) unless ($accepted);
      $val = $$var;
-     $changed{$opt} = $val if (!defined $old || !defined $val || $old ne $val);
+     $changed{$opt} = $val if (!defined $old || !defined $val || "$old" ne "$val");
     }
    $cw->Configured(\%args,\%changed);
   }
@@ -350,28 +312,29 @@ sub ConfigDefault
 {
  my ($cw,$args) = @_;
 
- $cw->BackTrace("Bad args") unless (defined $args && ref $args eq 'HASH');
+ $cw->BackTrace('Bad args') unless (defined $args && ref $args eq 'HASH');
 
  my $specs = $cw->ConfigSpecs;
  # Should we enforce a Delagates(DEFAULT => )  as well ?
  $specs->{'DEFAULT'} = ['SELF'] unless (exists $specs->{'DEFAULT'});
 
- # 
+ #
  # This is a pain with Text or Entry as core widget, they don't
  # inherit SELF's cursor. So comment it out for Tk402.001
- # 
+ #
  # $specs->{'-cursor'} = ['SELF',undef,undef,undef] unless (exists $specs->{'-cursor'});
 
- # Now some hacks that cause colours to propogate down a composite widget 
- # tree - really needs more thought, other options adding such as active 
+ # Now some hacks that cause colours to propogate down a composite widget
+ # tree - really needs more thought, other options adding such as active
  # colours too and maybe fonts
 
  my $children = scalar($cw->children);
+
  unless (exists($specs->{'-background'}))
   {
    my (@bg) = ('SELF');
    push(@bg,'CHILDREN') if $children;
-   $specs->{'-background'} = [\@bg,'background','Background',NORMAL_BG]; 
+   $specs->{'-background'} = [\@bg,'background','Background',NORMAL_BG];
   }
  unless (exists($specs->{'-foreground'}))
   {
@@ -383,7 +346,7 @@ sub ConfigDefault
 
  # Pre-scan args for aliases - this avoids defaulting
  # options specified via alias
- my $opt; 
+ my $opt;
  foreach $opt (keys %$args)
   {
    my $info = $specs->{$opt};
@@ -393,9 +356,9 @@ sub ConfigDefault
     }
   }
 
- # Now walk %$specs supplying defaults for all the options 
+ # Now walk %$specs supplying defaults for all the options
  # which have a defined default value, potentially looking up .Xdefaults database
- # options for the name/class of the 'frame' 
+ # options for the name/class of the 'frame'
 
  foreach $opt (keys %$specs)
   {
@@ -407,18 +370,21 @@ sub ConfigDefault
        if (ref $info)
         {
          # Not an alias
+         if ($ENHANCED_CONFIGSPECS && !defined $info->[3])
+          {
+           # configure inquire to fill in default slot from subwidget
+           $cw->configure($opt);
+          }
          if (defined $info->[3])
           {
-           # Only propagate if a default is supplied
-           $args->{$opt} = $info->[3];
-           # maybe should convert -fred into 'fred','Fred' here 
            if (defined $info->[1] && defined $info->[2])
             {
              # Should we do this on the Subconfigure widget instead?
-             # to match *Entry.Background 
+             # to match *Entry.Background
              my $db = $cw->optionGet($info->[1],$info->[2]);
-             $args->{$opt} = $db if (defined $db);
+             $info->[3] = $db if (defined $db);
             }
+           $args->{$opt} = $info->[3];
           }
         }
       }
@@ -426,9 +392,6 @@ sub ConfigDefault
   }
 }
 
-=item -E<gt>ConfigSpecs(-I<key> =E<gt> [I<kind>, I<name>, I<Class>, I<default], ...)
-
-=cut 
 
 sub ConfigSpecs
 {
@@ -466,15 +429,14 @@ sub ConfigAlias
     {
      $specs->{$opt}  = $main unless (exists $specs->{$opt});
     }
-   else 
+   else
     {
      $cw->BackTrace("Neither $opt nor $main exist");
     }
   }
- $cw->BackTrace("Odd number of args to ConfigAlias") if (@_);
+ $cw->BackTrace('Odd number of args to ConfigAlias') if (@_);
 }
 
-
 sub Delegate
 {
  my ($cw,$method,@args) = @_;
@@ -495,20 +457,20 @@ sub Delegate
 sub InitObject
 {
  my ($cw,$args) = @_;
- $cw->Populate($args);    
+ $cw->Populate($args);
  $cw->ConfigDefault($args);
 }
 
 sub ConfigChanged
 {
  my ($cw,$args) = @_;
-}       
+}
 
 sub Advertise
 {
  my ($cw,$name,$widget)  = @_;
- confess "No name" unless (defined $name);
- croak "No widget" unless (defined $widget);
+ confess 'No name' unless (defined $name);
+ croak 'No widget' unless (defined $widget);
  $cw->{SubWidget} = {} unless (exists $cw->{SubWidget});
  $cw->{SubWidget}{$name} = $widget;              # advertise it
  return $widget;
@@ -523,15 +485,11 @@ sub Component
  my $w = $cw->$kind(%args);            # Create it
  # $w->pack(@$pack) if (defined $pack);
  $cw->Advertise($name,$w) if (defined $name);
- $cw->Delegates(map(($_ => $w),@$delegate)) if (defined $delegate); 
+ $cw->Delegates(map(($_ => $w),@$delegate)) if (defined $delegate);
  return $w;                            # and return it
 }
 
 1;
 __END__
 
-=back
-
-=cut
-
 
