@@ -1,16 +1,16 @@
-# Copyright (c) 1995-1998 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Table;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = '3.004'; # $Id: //depot/Tk8/Tk/Table.pm#4$
+$VERSION = '3.016'; # $Id: //depot/Tk8/Tk/Table.pm#16 $
 
 use Tk::Pretty;
 use AutoLoader;
-require Tk::Frame;
-@Tk::Table::ISA = qw(Tk::Frame);
+use base qw(Tk::Frame);
+
 Construct Tk::Widget 'Table';
 
 sub ClassInit
@@ -56,7 +56,7 @@ sub FocusChildren
  return () if ($t->cget('-takefocus'));
  return $t->SUPER::FocusChildren;
 }
-                                        
+
 sub Populate
 {
  my ($t,$args) = @_;
@@ -102,7 +102,6 @@ sub sizeN
  return $max;
 }
 
-
 sub total
 {
  my ($a)   = @_;
@@ -125,7 +124,7 @@ sub constrain
  $n = $fixed if ($n < $fixed);
  for ($i= 0; $i < $fixed; $i++)
   {
-   $total += $a->[$i];
+    (defined($a->[$i])) && ($total += $a->[$i]);
   }
  for ($i=$n; $total < $pixels && $i < @$a; $i++)
   {
@@ -151,11 +150,11 @@ sub Layout
  my $fcols  = $t->cget(-fixedcolumns);
  my $sb     = $t->cget(-scrollbars);
  my $H      = $t->Height;
- my $W      = $t->Width; 
- my $tadj   = $bw; 
- my $badj   = $bw; 
- my $ladj   = $bw; 
- my $radj   = $bw; 
+ my $W      = $t->Width;
+ my $tadj   = $bw;
+ my $badj   = $bw;
+ my $ladj   = $bw;
+ my $radj   = $bw;
  my @xs     = ($W,0,0,0);
  my @ys     = (0,$H,0,0);
  my $xsb;
@@ -212,9 +211,9 @@ sub Layout
  my $top  = $t->{Top}+$frows;
  my $left = $t->{Left}+$fcols;
 
- if ($why & 49)  
+ if ($why & 49)
   {
-   # Width and/or Height of element or 
+   # Width and/or Height of element or
    # number of rows and/or columns or
    # scrollbar presence has changed
    my $w = sizeN($t->cget('-columns'),$t->{Width})+$radj+$ladj;
@@ -245,14 +244,14 @@ sub Layout
              $s->UnmapWindow;
              if ($why & 1)
               {
-               my $w = $t->{Width}[$c]; 
+               my $w = $t->{Width}[$c];
                $s->ResizeWindow($w,$h);
               }
             }
           }
         }
       }
-     else 
+     else
       {
        my $hwm  = $left-$fcols;
        my $sh   = 0;
@@ -262,7 +261,7 @@ sub Layout
        for ($c = 0; $c <$cols; $c++)
         {
          my $s = $t->{Row}[$r][$c];
-         my $w = $t->{Width}[$c]; 
+         my $w = $t->{Width}[$c];
          if (($c < $left && $c >= $fcols) || ($x+$w > $W-$radj) )
           {
            if (defined $s)
@@ -289,8 +288,8 @@ sub Layout
            $x     += $w;
            if ($c >= $fcols)
             {
-             $hwm++;      
-             $sh    += $w 
+             $hwm++;
+             $sh    += $w
             }
           }
         }
@@ -309,7 +308,7 @@ sub Layout
    if (defined $xsb && $xs[2] > 0)
     {
      $xsb->MoveResizeWindow(@xs);
-     $cols -= $fcols; 
+     $cols -= $fcols;
      if ($cols > 0)
       {
        $xsb->set($t->{Left}/$cols,$t->{Right}/$cols);
@@ -379,7 +378,7 @@ sub LostSlave
   }
  else
   {
-   $t->BackTrace("Cannot find" . $s->PathName);
+   $t->BackTrace('Cannot find' . $s->PathName);
   }
  $t->QueueLayout(2);
 }
@@ -391,8 +390,8 @@ sub put
  $t->ManageGeometry($w);
  unless (defined $t->{Row}[$row])
   {
-   $t->{Row}[$row] = []; 
-   $t->{Height}[$row] = 0; 
+   $t->{Row}[$row] = [];
+   $t->{Height}[$row] = 0;
   }
  unless (defined $t->{Width}[$col])
   {
@@ -536,83 +535,5 @@ sub see
  return $see;
 }
 
-
-=head1 NAME
-
-Tk::Table - Scrollable 2 dimensional table of Tk widgets
-
-=head1 SYNOPSIS
-
-  use Tk::Table;
-
-  $table = $parent->Table(-rows => number,
-                          -columns => number,
-                          -scrollbars => anchor,
-                          -fixedrows => number,
-                          -fixedcolumns => number,
-                          -takefocus => boolean);
-
-  $widget = $table->Button(...);
-
-  $old = $table->put($row,$col,$widget);
-  $old = $table->put($row,$col,"Text");  # simple Label 
-  $widget = $table->get($row,$col);
-
-  $cols = $table->totalColumns;
-  $rows = $table->totalRows;
-
-  $table->see($widget);
-  $table->see($row,$col);
-
-  ($row,$col) = $table->Posn($widget);
-
-=head1 DESCRIPTION 
-
-Tk::Table is an all-perl widget/geometry manager which allows a two dimensional
-table of arbitary perl/Tk widgets to be displayed.
-
-Entries in the Table are simply ordinary perl/Tk widgets. They should
-be created with the Table as their parent. Widgets are positioned in the 
-table using:
-
- $table->put($row,$col,$widget)
-
-All the widgets in each column are set to the same width - the requested
-width of the widest widget in the column.
-Likewise, all the widgets in each row are set to the same height - the requested
-height of the tallest widget in the column.             
-
-A number of rows and/or columns can be marked as 'fixed' - and so can serve
-as 'headings' for the remainder the rows which are scrollable.
-
-The requested size of the table as a whole is such that the number of rows
-specified by -rows (default 10), and number of columns specified by -columns
-(default 10) can be displayed.
-
-If the Table is told it can take the keyboard focus then cursor and scroll
-keys scroll the displayed widgets.
-
-The Table will create and manage its own scrollbars if requested via 
--scrollbars.
-
-The Tk::Table widget is derived from a Tk::Frame, so inherits all its
-configure options.
-
-=head1 BUGS / Snags / Possible enhancements
-
-=over 3
-
-=item * 
-
-Very large Tables consume a lot of X windows
-
-=item * 
-
-No equivalent of pack's -anchor/-pad etc. options 
-
-=back 
-
 =cut
 
-
-
