@@ -1,21 +1,18 @@
-# Copyright (c) 1995-1998 Nick Ing-Simmons. All rights reserved.
+# Copyright (c) 1995-1999 Nick Ing-Simmons. All rights reserved.
 # This program is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 package Tk::Widget;
+use vars qw($VERSION);
+$VERSION = '3.054'; # $Id: //depot/Tk8/Tk/Widget.pm#54 $
+
 require Tk;
 use AutoLoader;
-require DynaLoader;
 use strict;
-
 use Carp;
-
-@Tk::Widget::ISA = qw(DynaLoader Tk);
+use base qw(DynaLoader Tk);
 
 # stubs for 'autoloaded' widget classes
 
-use vars qw($VERSION);
-$VERSION = '3.015'; # $Id: //depot/Tk8/Tk/Widget.pm#15$
-
 sub Button;
 sub Canvas;
 sub Checkbutton;
@@ -37,18 +34,18 @@ sub Bitmap;
 sub Photo;
 
 sub ScrlListbox;
-sub Optionmenu; 
+sub Optionmenu;
 
 sub import
 {
  my $package = shift;
- carp "use Tk::Widget () to pre-load widgets is deprecated" if (@_ && $^W);
+ carp 'use Tk::Widget () to pre-load widgets is deprecated' if (@_ && $^W);
  my $need;
  foreach $need (@_)
   {
    unless (defined &{$need})
     {
-     require "Tk/${need}.pm"; 
+     require "Tk/${need}.pm";
     }
    croak "Cannot locate $need" unless (defined &{$need});
   }
@@ -64,24 +61,32 @@ use Tk::Submethods( 'grab' =>  [qw(current status release -global)],
                     'pack'  => [qw(configure forget info propagate slaves)],
                     'grid'  => [qw(bbox columnconfigure configure forget info location propagate rowconfigure size slaves)],
                     'form'  => [qw(check configure forget grid info slaves)],
-                    'after' => [qw(cancel idle)],
                     'event' => [qw(add delete generate info)],
                     'place' => [qw(configure forget info slaves)],
                     'wm'    => [qw(capture release)],
                     'font'  => [qw(actual configure create delete families measure metrics names)]
-                  );               
+                  );
+
+BEGIN  {
+ # FIXME - these don't work in the compiler
+ *IsMenu         = \&False;
+ *IsMenubutton   = \&False;
+ *configure_self = \&Tk::configure;
+ *cget_self      = \&Tk::cget;
+}
 
 
-*IsMenu       = \&False;
-*IsMenubutton = \&False;
 
-Direct Tk::Submethods ( 'winfo' => [qw(cells class colormapfull depth exists
+Direct Tk::Submethods (
+  'winfo' => [qw(cells class colormapfull depth exists
                geometry height id ismapped manager name parent reqheight
                reqwidth rootx rooty screen screencells screendepth screenheight
                screenmmheight screenmmwidth  screenvisual screenwidth visual
                visualsavailable  vrootheight viewable vrootwidth vrootx vrooty
                width x y toplevel children pixels pointerx pointery pointerxy
-               server fpixels rgb )]);
+               server fpixels rgb )],
+   'tk'   => [qw(appname scaling)]);
+
 
 sub DESTROY
 {
@@ -89,10 +94,10 @@ sub DESTROY
  $w->destroy if ($w->IsWidget);
 }
 
-sub Install 
+sub Install
 {
- # Dynamically loaded widgets add their core commands 
- # to the Tk base class here 
+ # Dynamically loaded widgets add their core commands
+ # to the Tk base class here
  my ($package,$mw) = @_;
 }
 
@@ -103,28 +108,38 @@ sub ClassInit
  return $package;
 }
 
+sub CreateOptions
+{
+ return ();
+}
+
 sub CreateArgs
 {
  my ($package,$parent,$args) = @_;
  # Remove from hash %$args any configure-like
- # options which only apply at create time (e.g. -colormap for Frame)
+ # options which only apply at create time (e.g. -colormap for Frame),
+ # or which may as well be applied right away
  # return these as a list of -key => value pairs
  # Augment same hash with default values for missing mandatory options,
  # allthough this can be done later in InitObject.
 
- # Honour -class => if present, we have hacked Tk_ConfigureWidget to 
- # allow -class to be passed to any widget.                         
+ # Honour -class => if present, we have hacked Tk_ConfigureWidget to
+ # allow -class to be passed to any widget.
  my @result = ();
- my $class = delete $args->{'-class'};                     
+ my $class = delete $args->{'-class'};
  ($class) = $package =~ /([A-Z][A-Z0-9_]*)$/i unless (defined $class);
  push(@result, '-class' => "\u$class") if (defined $class);
+ foreach my $opt ($package->CreateOptions)
+  {
+   push(@result, $opt => delete $args->{$opt}) if exists $args->{$opt};
+  }
  return @result;
 }
 
 sub InitObject
 {
  my ($obj,$args) = @_;
- # per object initialization, for example populating 
+ # per object initialization, for example populating
  # with sub-widgets, adding a few object bindings to augment
  # inherited class bindings, changing binding tags.
  # Also another chance to mess with %$args before configure...
@@ -148,7 +163,7 @@ sub new
  my $cmd   = $package->Tk_cmd;
  my $pname = $parent->PathName;
  $pname    = '' if ($pname eq '.');
- my $leaf  = delete $args{'Name'};          
+ my $leaf  = delete $args{'Name'};
  if (defined $leaf)
   {
    $leaf =~ s/[^a-z0-9_]+/_/ig;
@@ -158,22 +173,26 @@ sub new
   {
    ($leaf) = "\L$package" =~ /([a-z][a-z0-9_]*)$/;
   }
- my $lname  = $pname . "." . $leaf;
- # create a hash indexed by leaf name to speed up   
+ my $lname  = $pname . '.' . $leaf;
+ # create a hash indexed by leaf name to speed up
  # creation of a lot of sub-widgets of the same type
  # e.g. entries in Table
  my $key = "_#$leaf";
  $parent->{$key} = 0 unless (exists $parent->{$key});
  while (defined ($parent->Widget($lname)))
   {
-   $lname = $pname . "." . $leaf . ++$parent->{$key};
+   $lname = $pname . '.' . $leaf . ++$parent->{$key};
   }
  my $obj = eval { &$cmd($parent, $lname, @args) };
  confess $@ if $@;
  bless $obj,$package;
- $obj->InitObject(\%args);
- $obj->configure(%args) if (%args);
  $obj->SetBindtags;
+ my $notice = $parent->can('NoticeChild');
+ $parent->$notice($obj,\%args) if $notice;
+ $obj->InitObject(\%args);
+# ASkludge(\%args,1);
+ $obj->configure(%args) if (%args);
+# ASkludge(\%args,0);
  return $obj;
 }
 
@@ -243,7 +262,7 @@ sub AUTOLOAD
  # Braces used to preserve $1 et al.
  {
   my ($pkg,$func) = $what =~ /(.*)::([^:]+)$/;
-  confess("Attempt to load '$what'") unless defined $pkg;
+  confess("Attempt to load '$what'") unless defined($pkg) && $func =~ /^[\w:]+$/;
   $pkg =~ s#::#/#g;
   if (defined($name=$INC{"$pkg.pm"}))
    {
@@ -260,44 +279,44 @@ sub AUTOLOAD
  eval {local $SIG{'__DIE__'}; require $name};
  if ($@)
   {
-   croak $@ unless ($@ =~ /Can't locate \Q$name\E/);
+   croak $@ unless ($@ =~ /Can't locate\s+(?:file\s+)?'?\Q$name\E'?/);
    my($package,$method) = ($what =~ /^(.*)::([^:]*)$/);
    if ($package eq 'Tk::Widget' && $method ne '__ANON__')
     {
      # carp "Assuming 'require Tk::$method;'" if ($^W);
      require "Tk/$method.pm";
     }
-   else 
+   else
     {
      if (ref $_[0] && $method !~ /^(ConfigSpecs|Delegates)/ )
-      {                                    
-       my $delegate = $_[0]->Delegates;    
-       if (%$delegate || tied %$delegate)                     
-        {                                  
+      {
+       my $delegate = $_[0]->Delegates;
+       if (%$delegate || tied %$delegate)
+        {
          my $widget = $delegate->{$method};
          $widget = $delegate->{DEFAULT} unless (defined $widget);
-         if (defined $widget)              
-          {                                
+         if (defined $widget)
+          {
            my $subwidget = (ref $widget) ? $widget : $_[0]->Subwidget($widget);
-           if (defined $subwidget)         
-            {                              
+           if (defined $subwidget)
+            {
              no strict 'refs';
              # print "AUTOLOAD: $what\n";
-             *{$what} = sub { shift->Delegate($method,@_) }; 
-            }                              
-           else                            
-            {                              
+             *{$what} = sub { shift->Delegate($method,@_) };
+            }
+           else
+            {
              croak "No delegate subwidget '$widget' for $what";
-            }                              
-          }                                
-        }                                  
+            }
+          }
+        }
        if (!defined(&$what) && $method =~ /^[A-Z]\w+$/ && ref($_[0]) && $_[0]->isa('Tk::Widget'))
         {
          $what = "Tk::Widget::$method";
          carp "Assuming 'require Tk::$method;'" if ($^W);
          require "Tk/$method.pm";
         }
-      }                                    
+      }
     }
   }
  $@ = $save;
@@ -305,13 +324,8 @@ sub AUTOLOAD
  goto &$what;
 }
 
-*isa = \&True if ($] <= 5.003);
-
-*configure_self = \&Tk::configure;
-*cget_self = \&Tk::cget;
-
 sub _Destroyed
-{ 
+{
  my $w = shift;
  my $a = delete $w->{'_Destroy_'};
  return unless ref $a;
@@ -342,20 +356,21 @@ sub ImageMethod
   }
 }
 
-sub Getimage 
+sub Getimage
 {
  my ($w, $name) = @_;
  my $mw = $w->MainWindow;
+ croak "Usage \$widget->Getimage('name')" unless defined($name);
  my $images = ($mw->{'__Images__'} ||= {});
 
  return $images->{$name} if $images->{$name};
 
- ImageMethod(xpm => "Pixmap",
-    gif => "Photo",
-    ppm => "Photo",
-    xbm => "Bitmap" ) unless @image_types;
+ ImageMethod(xpm => 'Pixmap',
+    gif => 'Photo',
+    ppm => 'Photo',
+    xbm => 'Bitmap' ) unless @image_types;
 
- foreach my $type (@image_types) 
+ foreach my $type (@image_types)
   {
    my $method = $image_method{$type};
    my $file = Tk->findINC( "$name.$type" );
@@ -369,8 +384,8 @@ sub Getimage
  return $images->{$name};
 }
 
-sub SaveGrabInfo 
-{               
+sub SaveGrabInfo
+{
  my $w = shift;
  $Tk::oldGrab = $w->grabCurrent;
  if (defined $Tk::oldGrab)
@@ -379,7 +394,6 @@ sub SaveGrabInfo
   }
 }
 
-
 sub grabSave
 {
  my ($w) = @_;
@@ -392,7 +406,7 @@ sub grabSave
 sub focusCurrent
 {
  my ($w) = @_;
- $w->Tk::focus('-displayof'); 
+ $w->Tk::focus('-displayof');
 }
 
 sub focusSave
@@ -413,27 +427,39 @@ sub OnDestroy
 # This is supposed to replicate Tk::after behaviour,
 # but does auto-cancel when widget is deleted.
 
+sub afterIdle
+{
+ require Tk::After;
+ my $w = shift;
+ return Tk::After->new($w,'idle','once',@_);
+}
+
+sub afterCancel
+{
+ my ($w,$what) = @_;
+ if (defined $what)
+  {
+   return $what->cancel if ref($what);
+   carp "dubious cancel of $what" if 0 && $^W;
+   $w->Tk::after('cancel' => $what);
+  }
+}
 
 sub after
 {
- require Tk::After;
  my $w = shift;
  my $t = shift;
  if (@_)
   {
-   return Tk::After->new($w,$t,'once',@_) if ($t ne 'cancel');
+   if ($t ne 'cancel')
+    {
+     require Tk::After;                     
+     return Tk::After->new($w,$t,'once',@_) 
+    }
    while (@_)
     {
      my $what = shift;
-     if (ref $what)
-      {
-       $what->cancel;
-      }
-     else
-      {
-       carp "dubious cancel of $what";
-       $w->Tk::after('cancel' => $what);
-      }
+     $w->afterCancel($what);
     }
   }
  else
@@ -450,37 +476,13 @@ sub repeat
  return Tk::After->new($w,$t,'repeat',@_);
 }
 
-sub Inherit
-{
- carp "Inherit is deprecated - use SUPER::";
- my $w = shift;
- my $method = shift;
- my ($class) = caller;
- *{$class.'::Inherit::ISA'} = \@{$class.'::ISA'} unless (defined @{$class.'::Inherit::ISA'});
- $class .= '::Inherit::';
- $class .= $method;
- return $w->$class(@_);
-}
-
-sub InheritThis
-{
- carp "InheritThis is deprecated - use SUPER::";
- my $w      = shift;
- my $what   = (caller(1))[3];
- my ($class,$method) = $what =~ /^(.*)::([^:]+)$/;
- *{$class.'::Inherit::ISA'} = \@{$class.'::ISA'} unless (defined @{$class.'::Inherit::ISA'});
- $class .= '::Inherit::';
- $class .= $method;
- return $w->$class(@_);
-}
-
 sub FindMenu
 {
- # default FindMenu is that there no menu.
+ # default FindMenu is that there is no menu.
  return undef;
 }
 
-sub XEvent { shift->{"_XEvent_"} }
+sub XEvent { shift->{'_XEvent_'} }
 
 sub propertyRoot
 {
@@ -488,7 +490,7 @@ sub propertyRoot
  return $w->property(@_,'root');
 }
 
-# atom, atomname, containing, interps, pathname 
+# atom, atomname, containing, interps, pathname
 # don't work this way - there is no window arg
 # So we pretend there was an call the C versions from Tk.xs
 
@@ -496,7 +498,6 @@ sub atom       { shift->InternAtom(@_)  }
 sub atomname   { shift->GetAtomName(@_) }
 sub containing { shift->Containing(@_)  }
 
-
 # interps not done yet
 # pathname not done yet
 
@@ -507,12 +508,12 @@ sub containing { shift->Containing(@_)  }
 # Walk should possibly be enhanced so allow early termination
 # like '-prune' of find.
 
-sub Walk 
+sub Walk
 {
  # Traverse a widget hierarchy while executing a subroutine.
  my($cw, $proc, @args) = @_;
  my $subwidget;
- foreach $subwidget ($cw->children) 
+ foreach $subwidget ($cw->children)
   {
    $subwidget->Walk($proc,@args);
    &$proc($subwidget, @args);
@@ -522,20 +523,20 @@ sub Walk
 sub Descendants
 {
  # Return a list of widgets derived from a parent widget and all its
- # descendants of a particular class.  
+ # descendants of a particular class.
  # If class is not passed returns the entire widget hierarchy.
- 
+
  my($widget, $class) = @_;
  my(@widget_tree)    = ();
- 
+
  $widget->Walk(
                sub { my ($widget,$list,$class) = @_;
                      push(@$list, $widget) if  (!defined($class) or $class eq $widget->class);
-                   }, 
+                   },
                \@widget_tree, $class
               );
  return @widget_tree;
-} 
+}
 
 sub Palette
 {
@@ -546,18 +547,18 @@ sub Palette
    my $c = $w->Checkbutton();
    my $e = $w->Entry();
    my $s = $w->Scrollbar();
-   $Palette{"activeBackground"}    = ($c->configure("-activebackground"))[3] ;
-   $Palette{"activeForeground"}    = ($c->configure("-activeforeground"))[3];
-   $Palette{"background"}          = ($c->configure("-background"))[3];
-   $Palette{"disabledForeground"}  = ($c->configure("-disabledforeground"))[3];
-   $Palette{"foreground"}          = ($c->configure("-foreground"))[3];
-   $Palette{"highlightBackground"} = ($c->configure("-highlightbackground"))[3];
-   $Palette{"highlightColor"}      = ($c->configure("-highlightcolor"))[3];
-   $Palette{"insertBackground"}    = ($e->configure("-insertbackground"))[3];
-   $Palette{"selectColor"}         = ($c->configure("-selectcolor"))[3];
-   $Palette{"selectBackground"}    = ($e->configure("-selectbackground"))[3];
-   $Palette{"selectForeground"}    = ($e->configure("-selectforeground"))[3];
-   $Palette{"troughColor"}         = ($s->configure("-troughcolor"))[3];
+   $Palette{'activeBackground'}    = ($c->configure('-activebackground'))[3] ;
+   $Palette{'activeForeground'}    = ($c->configure('-activeforeground'))[3];
+   $Palette{'background'}          = ($c->configure('-background'))[3];
+   $Palette{'disabledForeground'}  = ($c->configure('-disabledforeground'))[3];
+   $Palette{'foreground'}          = ($c->configure('-foreground'))[3];
+   $Palette{'highlightBackground'} = ($c->configure('-highlightbackground'))[3];
+   $Palette{'highlightColor'}      = ($c->configure('-highlightcolor'))[3];
+   $Palette{'insertBackground'}    = ($e->configure('-insertbackground'))[3];
+   $Palette{'selectColor'}         = ($c->configure('-selectcolor'))[3];
+   $Palette{'selectBackground'}    = ($e->configure('-selectbackground'))[3];
+   $Palette{'selectForeground'}    = ($e->configure('-selectforeground'))[3];
+   $Palette{'troughColor'}         = ($s->configure('-troughcolor'))[3];
    $c->destroy;
    $e->destroy;
    $s->destroy;
@@ -566,7 +567,6 @@ sub Palette
  return $w->{_Palette_};
 }
 
-
 # tk_setPalette --
 # Changes the default color scheme for a Tk application by setting
 # default colors in the option database and by modifying all of the
@@ -587,22 +587,22 @@ sub setPalette
  # Create an array that has the complete new palette. If some colors
  # aren't specified, compute them from other colors that are specified.
 
- die "must specify a background color" if (!exists $new{background});
- $new{"foreground"} = "black" unless (exists $new{foreground});
- my @bg = $w->rgb($new{"background"});
- my @fg = $w->rgb($new{"foreground"});
- my $darkerBg = sprintf("#%02x%02x%02x",9*$bg[0]/2560,9*$bg[1]/2560,9*$bg[2]/2560);
- foreach my $i ("activeForeground","insertBackground","selectForeground","highlightColor")
+ die 'must specify a background color' if (!exists $new{background});
+ $new{'foreground'} = 'black' unless (exists $new{foreground});
+ my @bg = $w->rgb($new{'background'});
+ my @fg = $w->rgb($new{'foreground'});
+ my $darkerBg = sprintf('#%02x%02x%02x',9*$bg[0]/2560,9*$bg[1]/2560,9*$bg[2]/2560);
+ foreach my $i ('activeForeground','insertBackground','selectForeground','highlightColor')
   {
-   $new{$i} = $new{"foreground"} unless (exists $new{$i});
+   $new{$i} = $new{'foreground'} unless (exists $new{$i});
   }
- unless (exists $new{"disabledForeground"})
+ unless (exists $new{'disabledForeground'})
   {
-   $new{"disabledForeground"} = sprintf("#%02x%02x%02x",(3*$bg[0]+$fg[0])/1024,(3*$bg[1]+$fg[1])/1024,(3*$bg[2]+$fg[2])/1024);
+   $new{'disabledForeground'} = sprintf('#%02x%02x%02x',(3*$bg[0]+$fg[0])/1024,(3*$bg[1]+$fg[1])/1024,(3*$bg[2]+$fg[2])/1024);
   }
- $new{"highlightBackground"} = $new{"background"} unless (exists $new{"highlightBackground"});
+ $new{'highlightBackground'} = $new{'background'} unless (exists $new{'highlightBackground'});
 
- unless (exists $new{"activeBackground"})
+ unless (exists $new{'activeBackground'})
   {
    my @light;
    # Pick a default active background that is lighter than the
@@ -624,11 +624,11 @@ sub setPalette
       }
      $light[$i] = 255 if ($light[$i] > 255);
     }
-   $new{"activeBackground"} = sprintf("#%02x%02x%02x",@light);
+   $new{'activeBackground'} = sprintf('#%02x%02x%02x',@light);
   }
- $new{"selectBackground"} = $darkerBg unless (exists $new{"selectBackground"});
- $new{"troughColor"} = $darkerBg unless (exists $new{"troughColor"});
- $new{"selectColor"} = "#b03060" unless (exists $new{"selectColor"});
+ $new{'selectBackground'} = $darkerBg unless (exists $new{'selectBackground'});
+ $new{'troughColor'} = $darkerBg unless (exists $new{'troughColor'});
+ $new{'selectColor'} = '#b03060' unless (exists $new{'selectColor'});
 
  # Before doing this, make sure that the Tk::Palette variable holds
  # the default values of all options, so that tkRecolorTree can
@@ -644,7 +644,7 @@ sub setPalette
  # same colors.
  foreach my $option (keys %new)
   {
-   $w->option("add","*$option",$new{$option},$priority);
+   $w->option('add',"*$option",$new{$option},$priority);
    # Save the options in the global variable Tk::Palette, for use the
    # next time we change the options.
    $Palette->{$option} = $new{$option};
@@ -683,7 +683,7 @@ sub RecolorTree
     }
   }
  foreach my $child ($w->children)
-  {                
+  {
    $child->RecolorTree($colors);
   }
 }
@@ -709,7 +709,7 @@ sub Darken
  $green = 255 if ($green > 255);
  $blue = int($blue*$percent/100);
  $blue = 255 if ($blue > 255);
- sprintf("#%02x%02x%02x",$red,$green,$blue)
+ sprintf('#%02x%02x%02x',$red,$green,$blue)
 }
 # tk_bisque --
 # Reset the Tk color palette to the old "bisque" colors.
@@ -718,18 +718,18 @@ sub Darken
 # None.
 sub bisque
 {
- shift->setPalette("activeBackground" => "#e6ceb1",
-               "activeForeground" => "black",
-               "background" => "#ffe4c4",
-               "disabledForeground" => "#b0b0b0",
-               "foreground" => "black",
-               "highlightBackground" => "#ffe4c4",
-               "highlightColor" => "black",
-               "insertBackground" => "black",
-               "selectColor" => "#b03060",
-               "selectBackground" => "#e6ceb1",
-               "selectForeground" => "black",
-               "troughColor" => "#cdb79e"
+ shift->setPalette('activeBackground' => '#e6ceb1',
+               'activeForeground' => 'black',
+               'background' => '#ffe4c4',
+               'disabledForeground' => '#b0b0b0',
+               'foreground' => 'black',
+               'highlightBackground' => '#ffe4c4',
+               'highlightColor' => 'black',
+               'insertBackground' => 'black',
+               'selectColor' => '#b03060',
+               'selectBackground' => '#e6ceb1',
+               'selectForeground' => 'black',
+               'troughColor' => '#cdb79e'
               );
 }
 
@@ -742,51 +742,93 @@ sub PrintConfig
   {
    print Tk::Pretty::Pretty(@$c),"\n";
   }
-} 
+}
+
+sub BusyRecurse
+{
+ my ($restore,$w,$cursor,$recurse,$top) = @_;
+ my $c = $w->cget('-cursor');
+ my @tags = $w->bindtags;
+ if ($top || defined($c))
+  {
+   push(@$restore, sub { $w->configure(-cursor => $c); $w->bindtags(\@tags) });
+   $w->configure(-cursor => $cursor);
+  }
+ else
+  {
+   push(@$restore, sub { $w->bindtags(\@tags) });
+  }
+ $w->bindtags(['Busy',@tags]);
+ if ($recurse)
+  {
+   foreach my $child ($w->children)
+    {
+     BusyRecurse($restore,$child,$cursor,1,0);
+    }
+  }
+ return $restore;
+}
 
 sub Busy
 {
  my ($w,%args) = @_;
  return unless $w->viewable;
- $args{'-cursor'} = 'watch' unless (exists $args{'-cursor'});
+ my $cursor  = delete $args{'-cursor'};
+ my $recurse = delete $args{'-recurse'};
+ $cursor  = 'watch' unless defined $cursor;
  unless (exists $w->{'Busy'})
   {
-   my %old = ();           
-   my $key;                
-   my @tags = $w->bindtags;
+   my @old = ($w->grabSave);
+   my $key;
+   my @config;
    foreach $key (keys %args)
     {
-     $old{$key} = $w->Tk::cget($key);
+     push(@config,$key => $w->Tk::cget($key));
+    }
+   if (@config)
+    {
+     push(@old, sub { $w->Tk::configure(@config) });
+     $w->Tk::configure(%args);
     }
-   $old{'bindtags'} = \@tags;
-   $old{'grab'}     = $w->grabSave;
    unless ($w->Tk::bind('Busy'))
-    {                     
-     $w->Tk::bind('Busy','<KeyPress>','bell');
-     $w->Tk::bind('Busy','<ButtonPress>','bell');
-    }                     
-   $w->bindtags(['Busy']);
-   $w->{'Busy'} = \%old;
+    {
+     $w->Tk::bind('Busy','<Any-KeyPress>',[_busy => 1]);
+     $w->Tk::bind('Busy','<Any-KeyRelease>',[_busy => 0]);
+     $w->Tk::bind('Busy','<Any-ButtonPress>',[_busy => 1]);
+     $w->Tk::bind('Busy','<Any-ButtonRelease>',[_busy => 0]);
+    }
+   $w->{'Busy'} = BusyRecurse(\@old,$w,$cursor,$recurse,1);
   }
- $w->Tk::configure(%args);
+ my $g = $w->grabCurrent;
+ if (defined $g)
+  {
+   # warn "$g has the grab";
+   $g->grabRelease;
+  }
+ $w->update;
  eval {local $SIG{'__DIE__'};  $w->grab };
  $w->update;
 }
 
+sub _busy
+{
+ my ($w,$f) = @_;
+ $w->bell if $f;
+ $w->break;
+}
+
 sub Unbusy
 {
  my ($w) = @_;
+ $w->update;
  $w->grabRelease;
  my $old = delete $w->{'Busy'};
  if (defined $old)
   {
-   my $grab = delete $old->{'grab'};
-   $w->update;  # flush events that happened with Busy bindings
-   $w->bindtags(delete $old->{'bindtags'});
-   $w->Tk::configure(%{$old}); 
-   $w->update;
-   &$grab;
+   local $SIG{'__DIE__'};
+   eval { &{pop(@$old)} } while (@$old);
   }
+ $w->update;
 }
 
 sub waitVisibility
@@ -872,7 +914,7 @@ sub XYscrollBind
 
 sub ScrlListbox
 {
- my $parent = shift; 
+ my $parent = shift;
  return $parent->Scrolled('Listbox',-scrollbars => 'w', @_);
 }
 
@@ -936,7 +978,7 @@ sub place
  else
   {
    # Two things going on here:
-   # 1. Add configure on the front so that we can drop leading '-' 
+   # 1. Add configure on the front so that we can drop leading '-'
    $w->Tk::place('configure',@_);
    # 2. Return the widget rather than nothing
    return $w;
@@ -954,7 +996,7 @@ sub pack
  else
   {
    # Two things going on here:
-   # 1. Add configure on the front so that we can drop leading '-' 
+   # 1. Add configure on the front so that we can drop leading '-'
    $w->Tk::pack('configure',@_);
    # 2. Return the widget rather than nothing
    return $w;
@@ -973,7 +1015,7 @@ sub grid
  else
   {
    # Two things going on here:
-   # 1. Add configure on the front so that we can drop leading '-' 
+   # 1. Add configure on the front so that we can drop leading '-'
    Tk::grid('configure',$w,@_);
    # 2. Return the widget rather than nothing
    return $w;
@@ -991,7 +1033,7 @@ sub form
  else
   {
    # Two things going on here:
-   # 1. Add configure on the front so that we can drop leading '-' 
+   # 1. Add configure on the front so that we can drop leading '-'
    $w->Tk::form('configure',@_);
    # 2. Return the widget rather than nothing
    return $w;
@@ -1001,24 +1043,27 @@ sub form
 sub Scrolled
 {
  my ($parent,$kind,%args) = @_;
+ # Find args that are Frame create time args
  my @args = Tk::Frame->CreateArgs($parent,\%args);
  my $name = delete $args{'Name'};
  push(@args,'Name' => $name) if (defined $name);
  my $cw = $parent->Frame(@args);
  @args = ();
- my $k;
- # Need to consider other 'Frame' configure options...
- foreach $k ('-scrollbars',map($_->[0],$cw->configure))
+ # Now remove any args that Frame can handle
+ foreach my $k ('-scrollbars',map($_->[0],$cw->configure))
   {
    push(@args,$k,delete($args{$k})) if (exists $args{$k})
   }
- $cw->ConfigSpecs('-scrollbars' => ['METHOD','scrollbars','Scrollbars','se'],
-                  '-background' => ['CHILDREN','background','Background',undef], 
-                 );
+ # Anything else must be for target widget - pass at widget create time
  my $w  = $cw->$kind(%args);
+ # Now re-set %args to be ones Frame can handle
  %args = @args;
+ $cw->ConfigSpecs('-scrollbars' => ['METHOD','scrollbars','Scrollbars','se'],
+                  '-background' => [$w,'background','Background'],
+                  '-foreground' => [$w,'foreground','Foreground'],
+                 );
  $cw->AddScrollbars($w);
- $cw->Default("\L$kind" => $w); 
+ $cw->Default("\L$kind" => $w);
  $cw->Delegates('bind' => $w, 'bindtags' => $w);
  $cw->ConfigDefault(\%args);
  $cw->configure(%args);
@@ -1036,7 +1081,7 @@ sub ForwardEvent
  my $to   = shift;
  $to->PassEvent($self->XEvent);
 }
- 
+
 # Save / Return abstract event type as in Tix.
 sub EventType
 {
@@ -1045,10 +1090,33 @@ sub EventType
  return $w->{'_EventType_'};
 }
 
-1; 
-
+1;
 __END__
 
+sub ASkludge
+{
+ my ($hash,$sense) = @_;
+ foreach my $key (%$hash)
+  {
+   if ($key =~ /-.*variable/ && ref($hash->{$key}) eq 'SCALAR')
+    {
+     if ($sense)
+      {
+       my $val = ${$hash->{$key}};
+       require Tie::Scalar;
+       tie ${$hash->{$key}},'Tie::StdScalar';
+       ${$hash->{$key}} = $val;
+      }
+     else
+      {
+       untie ${$hash->{$key}};
+      }
+    }
+  }
+}
+
+
+
 # clipboardKeysyms --
 # This procedure is invoked to identify the keys that correspond to
 # the "copy", "cut", and "paste" functions for the clipboard.
@@ -1059,12 +1127,12 @@ __END__
 # cut - Name of the key used for the cut operation.
 # paste - Name of the key used for the paste operation.
 #
-# This method is obsolete use clipboardOperations and abstract 
+# This method is obsolete use clipboardOperations and abstract
 # event types instead. See Clipboard.pm and Mainwindow.pm
 
 sub clipboardKeysyms
 {
- my @class = (); 
+ my @class = ();
  my $mw    = shift;
  if (ref $mw)
   {
@@ -1087,52 +1155,16 @@ sub clipboardKeysyms
   }
  if (@_)
   {
-   my $paste = shift;                                                
+   my $paste = shift;
    $mw->Tk::bind(@class,"<$paste>",'clipboardPaste') if (defined $paste);
   }
-}        
-
-
-
-=head1 NAME 
-
-Tk::Widget - Base class of all widgets
-
-=head1 SYNOPSIS
-
-   package Tk::Whatever;
-   require Tk::Widget;
-   @ISA = qw(Tk::Widget);   
-   Construct Tk::Widget 'Whatever';
-
-   sub Tk_cmd { \&Tk::whatever }
-
-=head1 DESCRIPTION 
-
-The B<Tk::Widget> is an abstract base class for all Tk widgets.
-
-Generic methods available to all widgets include:
-
-=over 4
-
-=item Getimage( name )
-
-Given I<name>, look for an image file with that base name and return
-a C<Tk::Image>.  File extensions are tried in this order: F<xpm>,
-F<gif>, F<ppm>, F<xbm> until a valid iamge is found.  If no image is
-found, try a builtin image with that name.
-
-=item EventType( ?value? )
-
-Return or set the tixEvent variable C<option>.  Currently, the only
-C<option> used is "type".
-
-=back 
-
-=head1 CAVEATS
-
-The above documentaion on generic methods is catastrophically incomplete.
-
-=cut 
+}
+
+sub pathname
+{
+ my ($w,$id) = @_;
+ my $x = $w->winfo('pathname',-displayof  => oct($id));
+ return $x->PathName;
+}
 
 
