@@ -11,18 +11,10 @@
 #  define LangBadFile (*LangVptr->V_LangBadFile)
 #endif
 
-#ifndef LangCallbackArg
-#  define LangCallbackArg (*LangVptr->V_LangCallbackArg)
-#endif
-
 #ifndef LangCmpArg
 #  define LangCmpArg (*LangVptr->V_LangCmpArg)
 #endif
 
-#ifndef LangCmpCallback
-#  define LangCmpCallback (*LangVptr->V_LangCmpCallback)
-#endif
-
 #ifndef LangCmpOpt
 #  define LangCmpOpt (*LangVptr->V_LangCmpOpt)
 #endif
@@ -31,10 +23,6 @@
 #  define LangCopyArg (*LangVptr->V_LangCopyArg)
 #endif
 
-#ifndef LangCopyCallback
-#  define LangCopyCallback (*LangVptr->V_LangCopyCallback)
-#endif
-
 #ifndef LangDebug
 #  define LangDebug (*LangVptr->V_LangDebug)
 #endif
@@ -63,10 +51,6 @@
 #  define LangFreeArg (*LangVptr->V_LangFreeArg)
 #endif
 
-#ifndef LangFreeCallback
-#  define LangFreeCallback (*LangVptr->V_LangFreeCallback)
-#endif
-
 #ifndef LangFreeVar
 #  define LangFreeVar (*LangVptr->V_LangFreeVar)
 #endif
@@ -79,10 +63,6 @@
 #  define LangLibraryDir (*LangVptr->V_LangLibraryDir)
 #endif
 
-#ifndef LangMakeCallback
-#  define LangMakeCallback (*LangVptr->V_LangMakeCallback)
-#endif
-
 #ifndef LangMergeString
 #  define LangMergeString (*LangVptr->V_LangMergeString)
 #endif
@@ -151,6 +131,10 @@
 #  define Lang_BuildInImages (*LangVptr->V_Lang_BuildInImages)
 #endif
 
+#ifndef Lang_CallWithArgs
+#  define Lang_CallWithArgs (*LangVptr->V_Lang_CallWithArgs)
+#endif
+
 #ifndef Lang_CreateObject
 #  define Lang_CreateObject (*LangVptr->V_Lang_CreateObject)
 #endif
@@ -203,18 +187,10 @@
 #  define Lang_SplitString (*LangVptr->V_Lang_SplitString)
 #endif
 
-#ifndef TclServiceIdle
-#  define TclServiceIdle (*LangVptr->V_TclServiceIdle)
-#endif
-
 #ifndef Tcl_AddErrorInfo
 #  define Tcl_AddErrorInfo (*LangVptr->V_Tcl_AddErrorInfo)
 #endif
 
-#ifndef Tcl_AfterObjCmd
-#  define Tcl_AfterObjCmd (*LangVptr->V_Tcl_AfterObjCmd)
-#endif
-
 #ifndef Tcl_Alloc
 #  define Tcl_Alloc (*LangVptr->V_Tcl_Alloc)
 #endif
@@ -243,26 +219,6 @@
 #  define Tcl_ArgResult (*LangVptr->V_Tcl_ArgResult)
 #endif
 
-#ifndef Tcl_AsyncCreate
-#  define Tcl_AsyncCreate (*LangVptr->V_Tcl_AsyncCreate)
-#endif
-
-#ifndef Tcl_AsyncDelete
-#  define Tcl_AsyncDelete (*LangVptr->V_Tcl_AsyncDelete)
-#endif
-
-#ifndef Tcl_AsyncInvoke
-#  define Tcl_AsyncInvoke (*LangVptr->V_Tcl_AsyncInvoke)
-#endif
-
-#ifndef Tcl_AsyncMark
-#  define Tcl_AsyncMark (*LangVptr->V_Tcl_AsyncMark)
-#endif
-
-#ifndef Tcl_AsyncReady
-#  define Tcl_AsyncReady (*LangVptr->V_Tcl_AsyncReady)
-#endif
-
 #ifndef Tcl_BackgroundError
 #  define Tcl_BackgroundError (*LangVptr->V_Tcl_BackgroundError)
 #endif
@@ -271,10 +227,6 @@
 #  define Tcl_CallWhenDeleted (*LangVptr->V_Tcl_CallWhenDeleted)
 #endif
 
-#ifndef Tcl_CancelIdleCall
-#  define Tcl_CancelIdleCall (*LangVptr->V_Tcl_CancelIdleCall)
-#endif
-
 #ifndef Tcl_Close
 #  define Tcl_Close (*LangVptr->V_Tcl_Close)
 #endif
@@ -287,18 +239,6 @@
 #  define Tcl_CreateCommand (*LangVptr->V_Tcl_CreateCommand)
 #endif
 
-#ifndef Tcl_CreateEventSource
-#  define Tcl_CreateEventSource (*LangVptr->V_Tcl_CreateEventSource)
-#endif
-
-#ifndef Tcl_CreateExitHandler
-#  define Tcl_CreateExitHandler (*LangVptr->V_Tcl_CreateExitHandler)
-#endif
-
-#ifndef Tcl_CreateFileHandler
-#  define Tcl_CreateFileHandler (*LangVptr->V_Tcl_CreateFileHandler)
-#endif
-
 #ifndef Tcl_CreateInterp
 #  define Tcl_CreateInterp (*LangVptr->V_Tcl_CreateInterp)
 #endif
@@ -307,10 +247,6 @@
 #  define Tcl_CreateObjCommand (*LangVptr->V_Tcl_CreateObjCommand)
 #endif
 
-#ifndef Tcl_CreateTimerHandler
-#  define Tcl_CreateTimerHandler (*LangVptr->V_Tcl_CreateTimerHandler)
-#endif
-
 #ifndef Tcl_DStringAppend
 #  define Tcl_DStringAppend (*LangVptr->V_Tcl_DStringAppend)
 #endif
@@ -323,6 +259,10 @@
 #  define Tcl_DStringFree (*LangVptr->V_Tcl_DStringFree)
 #endif
 
+#ifndef Tcl_DStringGetResult
+#  define Tcl_DStringGetResult (*LangVptr->V_Tcl_DStringGetResult)
+#endif
+
 #ifndef Tcl_DStringInit
 #  define Tcl_DStringInit (*LangVptr->V_Tcl_DStringInit)
 #endif
@@ -355,22 +295,6 @@
 #  define Tcl_DeleteCommandFromToken (*LangVptr->V_Tcl_DeleteCommandFromToken)
 #endif
 
-#ifndef Tcl_DeleteEventSource
-#  define Tcl_DeleteEventSource (*LangVptr->V_Tcl_DeleteEventSource)
-#endif
-
-#ifndef Tcl_DeleteEvents
-#  define Tcl_DeleteEvents (*LangVptr->V_Tcl_DeleteEvents)
-#endif
-
-#ifndef Tcl_DeleteExitHandler
-#  define Tcl_DeleteExitHandler (*LangVptr->V_Tcl_DeleteExitHandler)
-#endif
-
-#ifndef Tcl_DeleteFileHandler
-#  define Tcl_DeleteFileHandler (*LangVptr->V_Tcl_DeleteFileHandler)
-#endif
-
 #ifndef Tcl_DeleteHashEntry
 #  define Tcl_DeleteHashEntry (*LangVptr->V_Tcl_DeleteHashEntry)
 #endif
@@ -383,18 +307,6 @@
 #  define Tcl_DeleteInterp (*LangVptr->V_Tcl_DeleteInterp)
 #endif
 
-#ifndef Tcl_DeleteTimerHandler
-#  define Tcl_DeleteTimerHandler (*LangVptr->V_Tcl_DeleteTimerHandler)
-#endif
-
-#ifndef Tcl_DoOneEvent
-#  define Tcl_DoOneEvent (*LangVptr->V_Tcl_DoOneEvent)
-#endif
-
-#ifndef Tcl_DoWhenIdle
-#  define Tcl_DoWhenIdle (*LangVptr->V_Tcl_DoWhenIdle)
-#endif
-
 #ifndef Tcl_DoubleResults
 #  define Tcl_DoubleResults (*LangVptr->V_Tcl_DoubleResults)
 #endif
@@ -411,14 +323,6 @@
 #  define Tcl_EventuallyFree (*LangVptr->V_Tcl_EventuallyFree)
 #endif
 
-#ifndef Tcl_Exit
-#  define Tcl_Exit (*LangVptr->V_Tcl_Exit)
-#endif
-
-#ifndef Tcl_Finalize
-#  define Tcl_Finalize (*LangVptr->V_Tcl_Finalize)
-#endif
-
 #ifndef Tcl_FirstHashEntry
 #  define Tcl_FirstHashEntry (*LangVptr->V_Tcl_FirstHashEntry)
 #endif
@@ -479,10 +383,6 @@
 #  define Tcl_GetResult (*LangVptr->V_Tcl_GetResult)
 #endif
 
-#ifndef Tcl_GetServiceMode
-#  define Tcl_GetServiceMode (*LangVptr->V_Tcl_GetServiceMode)
-#endif
-
 #ifndef Tcl_GetStringFromObj
 #  define Tcl_GetStringFromObj (*LangVptr->V_Tcl_GetStringFromObj)
 #endif
@@ -495,10 +395,6 @@
 #  define Tcl_GetVar2 (*LangVptr->V_Tcl_GetVar2)
 #endif
 
-#ifndef Tcl_HashStats
-#  define Tcl_HashStats (*LangVptr->V_Tcl_HashStats)
-#endif
-
 #ifndef Tcl_HideCommand
 #  define Tcl_HideCommand (*LangVptr->V_Tcl_HideCommand)
 #endif
@@ -551,6 +447,14 @@
 #  define Tcl_Merge (*LangVptr->V_Tcl_Merge)
 #endif
 
+#ifndef Tcl_NewBooleanObj
+#  define Tcl_NewBooleanObj (*LangVptr->V_Tcl_NewBooleanObj)
+#endif
+
+#ifndef Tcl_NewDoubleObj
+#  define Tcl_NewDoubleObj (*LangVptr->V_Tcl_NewDoubleObj)
+#endif
+
 #ifndef Tcl_NewIntObj
 #  define Tcl_NewIntObj (*LangVptr->V_Tcl_NewIntObj)
 #endif
@@ -559,6 +463,14 @@
 #  define Tcl_NewListObj (*LangVptr->V_Tcl_NewListObj)
 #endif
 
+#ifndef Tcl_NewLongObj
+#  define Tcl_NewLongObj (*LangVptr->V_Tcl_NewLongObj)
+#endif
+
+#ifndef Tcl_NewObj
+#  define Tcl_NewObj (*LangVptr->V_Tcl_NewObj)
+#endif
+
 #ifndef Tcl_NewStringObj
 #  define Tcl_NewStringObj (*LangVptr->V_Tcl_NewStringObj)
 #endif
@@ -583,10 +495,6 @@
 #  define Tcl_Preserve (*LangVptr->V_Tcl_Preserve)
 #endif
 
-#ifndef Tcl_QueueEvent
-#  define Tcl_QueueEvent (*LangVptr->V_Tcl_QueueEvent)
-#endif
-
 #ifndef Tcl_Read
 #  define Tcl_Read (*LangVptr->V_Tcl_Read)
 #endif
@@ -615,14 +523,6 @@
 #  define Tcl_Seek (*LangVptr->V_Tcl_Seek)
 #endif
 
-#ifndef Tcl_ServiceAll
-#  define Tcl_ServiceAll (*LangVptr->V_Tcl_ServiceAll)
-#endif
-
-#ifndef Tcl_ServiceEvent
-#  define Tcl_ServiceEvent (*LangVptr->V_Tcl_ServiceEvent)
-#endif
-
 #ifndef Tcl_SetAssocData
 #  define Tcl_SetAssocData (*LangVptr->V_Tcl_SetAssocData)
 #endif
@@ -651,10 +551,6 @@
 #  define Tcl_SetLongObj (*LangVptr->V_Tcl_SetLongObj)
 #endif
 
-#ifndef Tcl_SetMaxBlockTime
-#  define Tcl_SetMaxBlockTime (*LangVptr->V_Tcl_SetMaxBlockTime)
-#endif
-
 #ifndef Tcl_SetObjResult
 #  define Tcl_SetObjResult (*LangVptr->V_Tcl_SetObjResult)
 #endif
@@ -663,18 +559,10 @@
 #  define Tcl_SetResult (*LangVptr->V_Tcl_SetResult)
 #endif
 
-#ifndef Tcl_SetServiceMode
-#  define Tcl_SetServiceMode (*LangVptr->V_Tcl_SetServiceMode)
-#endif
-
 #ifndef Tcl_SetStringObj
 #  define Tcl_SetStringObj (*LangVptr->V_Tcl_SetStringObj)
 #endif
 
-#ifndef Tcl_SetTimer
-#  define Tcl_SetTimer (*LangVptr->V_Tcl_SetTimer)
-#endif
-
 #ifndef Tcl_SetVar
 #  define Tcl_SetVar (*LangVptr->V_Tcl_SetVar)
 #endif
@@ -687,10 +575,6 @@
 #  define Tcl_SetVarArg (*LangVptr->V_Tcl_SetVarArg)
 #endif
 
-#ifndef Tcl_Sleep
-#  define Tcl_Sleep (*LangVptr->V_Tcl_Sleep)
-#endif
-
 #ifndef Tcl_SprintfResult
 #  define Tcl_SprintfResult (*LangVptr->V_Tcl_SprintfResult)
 #endif
@@ -719,10 +603,6 @@
 #  define Tcl_UntraceVar2 (*LangVptr->V_Tcl_UntraceVar2)
 #endif
 
-#ifndef Tcl_WaitForEvent
-#  define Tcl_WaitForEvent (*LangVptr->V_Tcl_WaitForEvent)
-#endif
-
 #ifndef Tcl_Write
 #  define Tcl_Write (*LangVptr->V_Tcl_Write)
 #endif
@@ -731,9 +611,5 @@
 #  define Tcl_WrongNumArgs (*LangVptr->V_Tcl_WrongNumArgs)
 #endif
 
-#ifndef TclpGetTime
-#  define TclpGetTime (*LangVptr->V_TclpGetTime)
-#endif
-
 #endif /* NO_VTABLES */
 #endif /* _LANG_VM */
