@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkConfig.c 1.53 96/04/26 10:29:31
+ * RCS: @(#) $Id: tkConfig.c,v 1.2 1998/09/14 18:23:08 stanton Exp $
  */
 
 #include "tkPort.h"
@@ -125,7 +125,14 @@ Tk_ConfigureWidget(interp, tkwin, specs, argc, argv, widgRec, flags)
      */
 
     for ( ; argc > 0; argc -= 2, argv += 2) {
-	specPtr = FindConfigSpec(interp, specs, *argv, needFlags, hateFlags);
+	char *arg;
+
+	if (flags & TK_CONFIG_OBJS) {
+	    arg = Tcl_GetStringFromObj(*args, NULL);
+	} else {
+	    arg = *argv;
+	}
+	specPtr = FindConfigSpec(interp, specs, arg, needFlags, hateFlags);
 	if (specPtr == NULL) {
 	    if (!(flags & TK_CONFIG_ARGV_ONLY)) {
 		/*
@@ -148,10 +155,18 @@ Tk_ConfigureWidget(interp, tkwin, specs, argc, argv, widgRec, flags)
 	 */
 
 	if (argc < 2) {
-	    Tcl_AppendResult(interp, "value for \"", *argv,
+	    Tcl_AppendResult(interp, "value for \"", arg,
 		    "\" missing", (char *) NULL);
 	    return TCL_ERROR;
 	}
+#if 0 /* DASH PATCH */
+	if (flags & TK_CONFIG_OBJS) {
+	    arg = Tcl_GetStringFromObj(args[1], NULL);
+	} else {
+	    arg = argv[1];
+	}
+	if (DoConfig(interp, tkwin, specPtr, arg, 0, widgRec) != TCL_OK) { }
+#endif /* DASH PATCH */
 	if (DoConfig(interp, tkwin, specPtr, args[1], widgRec) != TCL_OK) {
 	    char msg[100];
 
@@ -491,8 +506,10 @@ DoConfig(interp, tkwin, specPtr, value, widgRec)
 
 		if (nullValue) {
 		    new = NULL;
-		} else {
-		    new = Tk_GetFontFromObj(interp, tkwin, value);
+		} else {   
+		    Arg tmp = LangCopyArg(value);
+		    new = Tk_GetFontFromObj(interp, tkwin, tmp);
+		    LangFreeArg(tmp, TCL_DYNAMIC);
 		    if (new == NULL) {
 			return TCL_ERROR;
 		    }
@@ -504,7 +521,9 @@ DoConfig(interp, tkwin, specPtr, value, widgRec)
 	    case TK_CONFIG_BITMAP: {
 		Pixmap new, old;
 
-		if (nullValue) {
+		if (nullValue || 
+		   (( specPtr->specFlags & TK_CONFIG_NULL_OK) && 
+                      !*LangString(value)))  {
 		    new = None;
 	        } else {
 		    uid = Tk_GetUid(LangString(value));
@@ -879,7 +898,7 @@ FormatConfigValue(interp, tkwin, specPtr, widgRec, freeProcPtr)
 	case TK_CONFIG_FONT: {
 	    Tk_Font tkfont = *((Tk_Font *) ptr);
 	    if (tkfont != NULL) {
-		LangSetString(&result,Tk_NameOfFont(tkfont));
+		LangSetArg(&result, LangFontArg(interp, tkfont, NULL));
 	    }
 	    break;
 	}
