@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMenu.c 1.148 97/10/29 09:22:00
+ * RCS: @(#) $Id: tkMenu.c,v 1.2 1998/09/14 18:23:14 stanton Exp $
  */
 
 /*
@@ -76,6 +76,28 @@
 static int menusInitialized;	/* Whether or not the hash tables, etc., have
 				 * been setup */
 
+/*
+ * Custom option for handling "-state" and "-tile"
+ */
+
+static Tk_CustomOption stateOption = {
+    Tk_StateParseProc,
+    Tk_StatePrintProc,
+    (ClientData) 1	/* allow "normal", "active" and "disabled" */
+};
+
+static Tk_CustomOption tileOption = {
+    Tk_TileParseProc,
+    Tk_TilePrintProc,
+    (ClientData) NULL
+};
+
+static Tk_CustomOption offsetOption = {
+    Tk_OffsetParseProc,
+    Tk_OffsetPrintProc,
+    (ClientData) NULL
+};
+
 /*
  * Configuration specs for individual menu entries. If this changes, be sure
  * to update code in TkpMenuInit that changes the font string entry.
@@ -90,14 +112,21 @@ Tk_ConfigSpec tkMenuEntryConfigSpecs[] = {
 	DEF_MENU_ENTRY_ACTIVE_FG, Tk_Offset(TkMenuEntry, activeFg),
 	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK
 	|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-activetile", "activeTile", "Tile", (char *) NULL,
+	Tk_Offset(TkMenuEntry, activeTile), COMMAND_MASK|CHECK_BUTTON_MASK|
+	RADIO_BUTTON_MASK|CASCADE_MASK|TK_CONFIG_DONT_SET_DEFAULT,
+	&tileOption},
     {TK_CONFIG_STRING, "-accelerator", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_ACCELERATOR, Tk_Offset(TkMenuEntry, accel),
 	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK
 	|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_BORDER, "-background", (char *) NULL, (char *) NULL,
+    {TK_CONFIG_BORDER, "-background", "background", "Background",
 	DEF_MENU_ENTRY_BG, Tk_Offset(TkMenuEntry, border),
 	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK
 	|SEPARATOR_MASK|TEAROFF_MASK|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_SYNONYM, "-bg", "background", (char *) NULL,
+	(char *) NULL, 0,  
+	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK},
     {TK_CONFIG_BITMAP, "-bitmap", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_BITMAP, Tk_Offset(TkMenuEntry, bitmap),
 	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK
@@ -109,6 +138,10 @@ Tk_ConfigSpec tkMenuEntryConfigSpecs[] = {
 	DEF_MENU_ENTRY_COMMAND, Tk_Offset(TkMenuEntry, command),
 	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK
 	|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-disabledtile", "disabledTile", "Tile", (char *) NULL,
+	Tk_Offset(TkMenuEntry, disabledTile), COMMAND_MASK|CHECK_BUTTON_MASK|
+	RADIO_BUTTON_MASK|CASCADE_MASK|TK_CONFIG_DONT_SET_DEFAULT,
+	&tileOption},
     {TK_CONFIG_FONT, "-font", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_FONT, Tk_Offset(TkMenuEntry, tkfont),
 	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK
@@ -134,6 +167,10 @@ Tk_ConfigSpec tkMenuEntryConfigSpecs[] = {
     {TK_CONFIG_LANGARG, "-menu", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_MENU, Tk_Offset(TkMenuEntry, name),
 	CASCADE_MASK|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", "0,0",
+	Tk_Offset(TkMenuEntry, tsoffset), COMMAND_MASK|CHECK_BUTTON_MASK|
+	RADIO_BUTTON_MASK|CASCADE_MASK|TK_CONFIG_DONT_SET_DEFAULT,
+	&offsetOption},
     {TK_CONFIG_LANGARG, "-offvalue", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_OFF_VALUE, Tk_Offset(TkMenuEntry, offValue),
 	CHECK_BUTTON_MASK},
@@ -146,10 +183,14 @@ Tk_ConfigSpec tkMenuEntryConfigSpecs[] = {
     {TK_CONFIG_OBJECT, "-selectimage", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_SELECT_IMAGE, Tk_Offset(TkMenuEntry, selectImageString),
 	CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|TK_CONFIG_NULL_OK},
-    {TK_CONFIG_UID, "-state", (char *) NULL, (char *) NULL,
+    {TK_CONFIG_CUSTOM, "-state", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_STATE, Tk_Offset(TkMenuEntry, state),
 	COMMAND_MASK|CHECK_BUTTON_MASK|RADIO_BUTTON_MASK|CASCADE_MASK
-	|TEAROFF_MASK|TK_CONFIG_DONT_SET_DEFAULT},
+	|TEAROFF_MASK|TK_CONFIG_DONT_SET_DEFAULT, &stateOption},
+    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
+	Tk_Offset(TkMenuEntry, tile), COMMAND_MASK|CHECK_BUTTON_MASK|
+	RADIO_BUTTON_MASK|CASCADE_MASK|TK_CONFIG_DONT_SET_DEFAULT,
+	&tileOption},
     {TK_CONFIG_LANGARG, "-value", (char *) NULL, (char *) NULL,
 	DEF_MENU_ENTRY_VALUE, Tk_Offset(TkMenuEntry, onValue),
 	RADIO_BUTTON_MASK|TK_CONFIG_NULL_OK},
@@ -188,6 +229,9 @@ Tk_ConfigSpec tkMenuConfigSpecs[] = {
     {TK_CONFIG_COLOR, "-activeforeground", "activeForeground", "Background",
 	DEF_MENU_ACTIVE_FG_MONO, Tk_Offset(TkMenu, activeFg),
 	TK_CONFIG_MONO_ONLY},
+    {TK_CONFIG_CUSTOM, "-activetile", "activeTile", "Tile", (char *) NULL,
+	Tk_Offset(TkMenu, activeTile), TK_CONFIG_DONT_SET_DEFAULT,
+	&tileOption},
     {TK_CONFIG_BORDER, "-background", "background", "Background",
 	DEF_MENU_BG_COLOR, Tk_Offset(TkMenu, border), TK_CONFIG_COLOR_ONLY},
     {TK_CONFIG_BORDER, "-background", "background", "Background",
@@ -206,12 +250,18 @@ Tk_ConfigSpec tkMenuConfigSpecs[] = {
     {TK_CONFIG_COLOR, "-disabledforeground", "disabledForeground",
 	"DisabledForeground", DEF_MENU_DISABLED_FG_MONO,
 	Tk_Offset(TkMenu, disabledFg), TK_CONFIG_MONO_ONLY|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-disabledtile", "disabledTile", "Tile", (char *) NULL,
+	Tk_Offset(TkMenu, disabledTile), TK_CONFIG_DONT_SET_DEFAULT,
+	&tileOption},
     {TK_CONFIG_SYNONYM, "-fg", "foreground", (char *) NULL,
 	(char *) NULL, 0, 0},
     {TK_CONFIG_FONT, "-font", "font", "Font",
 	DEF_MENU_FONT, Tk_Offset(TkMenu, tkfont), 0},
     {TK_CONFIG_COLOR, "-foreground", "foreground", "Foreground",
 	DEF_MENU_FG, Tk_Offset(TkMenu, fg), 0},
+    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", "0,0",
+	Tk_Offset(TkMenu, tsoffset), TK_CONFIG_DONT_SET_DEFAULT,
+	&offsetOption},
     {TK_CONFIG_CALLBACK, "-postcommand", "postCommand", "Command",
 	DEF_MENU_POST_COMMAND, Tk_Offset(TkMenu, postCommand), 
         TK_CONFIG_NULL_OK},
@@ -230,6 +280,8 @@ Tk_ConfigSpec tkMenuConfigSpecs[] = {
     {TK_CONFIG_CALLBACK, "-tearoffcommand", "tearOffCommand", "TearOffCommand",
 	DEF_MENU_TEAROFF_CMD, Tk_Offset(TkMenu, tearOffCommand),
 	TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
+	Tk_Offset(TkMenu, tile), TK_CONFIG_DONT_SET_DEFAULT, &tileOption},
     {TK_CONFIG_STRING, "-title", "title", "Title",
     	DEF_MENU_TITLE, Tk_Offset(TkMenu, title), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-type", "type", "Type",
@@ -397,6 +449,11 @@ Tk_MenuCmd(clientData, interp, argc, argv)
     menuPtr->parentTopLevelPtr = NULL;
     menuPtr->menuTypeName = NULL;
     menuPtr->title = NULL;
+    menuPtr->tile = menuPtr->activeTile = menuPtr->disabledTile = NULL;
+    menuPtr->tileGC = menuPtr->activeTileGC = menuPtr->disabledTileGC = None;
+    menuPtr->tsoffset.flags =  0;
+    menuPtr->tsoffset.xoffset =  0;
+    menuPtr->tsoffset.yoffset =  0;
     TkMenuInitializeDrawingFields(menuPtr);
 
     menuRefPtr = TkCreateMenuReferences(menuPtr->interp,
@@ -407,7 +464,7 @@ Tk_MenuCmd(clientData, interp, argc, argv)
     	goto error;
     }
 
-    Tk_SetClass(menuPtr->tkwin, "Menu");
+    TkClassOption(menuPtr->tkwin, "Menu",&argc,&argv);
     TkSetClassProcs(menuPtr->tkwin, &menuClass, (ClientData) menuPtr);
     Tk_CreateEventHandler(new, ExposureMask|StructureNotifyMask|ActivateMask,
 	    TkMenuEventProc, (ClientData) menuPtr);
@@ -573,7 +630,7 @@ MenuWidgetCmd(clientData, interp, argc, argv)
 	}
 	if (index >= 0) {
 	    if ((menuPtr->entries[index]->type == SEPARATOR_ENTRY)
-		    || (menuPtr->entries[index]->state == tkDisabledUid)) {
+		    || (menuPtr->entries[index]->state == TK_STATE_DISABLED)) {
 		index = -1;
 	    }
 	}
@@ -893,7 +950,7 @@ TkInvokeMenu(interp, menuPtr, index)
     	goto done;
     }
     mePtr = menuPtr->entries[index];
-    if (mePtr->state == tkDisabledUid) {
+    if (mePtr->state == TK_STATE_DISABLED) {
 	goto done;
     }
     Tcl_Preserve((ClientData) mePtr);
@@ -1021,6 +1078,15 @@ DestroyMenuInstance(menuPtr)
     if (menuPtr->entries != NULL) {
 	ckfree((char *) menuPtr->entries);
     }
+    if (menuPtr->tileGC != None) {
+	Tk_FreeGC(menuPtr->display, menuPtr->tileGC);
+    }
+    if (menuPtr->activeTileGC != None) {
+	Tk_FreeGC(menuPtr->display, menuPtr->activeTileGC);
+    }
+    if (menuPtr->disabledTileGC != None) {
+	Tk_FreeGC(menuPtr->display, menuPtr->disabledTileGC);
+    }
     TkMenuFreeDrawOptions(menuPtr);
     Tk_FreeOptions(tkMenuConfigSpecs, (char *) menuPtr, menuPtr->display, 0);
 
@@ -1216,6 +1282,12 @@ DestroyMenuEntry(memPtr)
     if (mePtr->selectImage != NULL) {
 	Tk_FreeImage(mePtr->selectImage);
     }
+    if (mePtr->tileGC != None) {
+	Tk_FreeGC(menuPtr->display, mePtr->tileGC);
+    }
+    if (mePtr->activeTileGC != None) {
+	Tk_FreeGC(menuPtr->display, mePtr->activeTileGC);
+    }
     if (mePtr->variable != NULL) {
 	Tcl_UntraceVar(menuPtr->interp, mePtr->variable,
 		TCL_GLOBAL_ONLY|TCL_TRACE_WRITES|TCL_TRACE_UNSETS,
@@ -1904,7 +1976,7 @@ MenuNewEntry(menuPtr, index, type)
     mePtr->selectImage = NULL;
     mePtr->accel = NULL;
     mePtr->accelLength = 0;
-    mePtr->state = tkNormalUid;
+    mePtr->state = TK_STATE_NORMAL;
     mePtr->border = NULL;
     mePtr->fg = NULL;
     mePtr->activeBorder = NULL;
@@ -1923,6 +1995,11 @@ MenuNewEntry(menuPtr, index, type)
     mePtr->entryFlags = 0;
     mePtr->index = index;
     mePtr->nextCascadePtr = NULL;
+    mePtr->tile = mePtr->activeTile = mePtr->disabledTile = NULL;
+    mePtr->tileGC = mePtr->activeTileGC = None;
+    mePtr->tsoffset.flags =  0;
+    mePtr->tsoffset.xoffset =  0;
+    mePtr->tsoffset.yoffset =  0;
     TkMenuInitializeEntryDrawingFields(mePtr);
     if (TkpMenuNewEntry(mePtr) != TCL_OK) {
     	ckfree((char *) mePtr);
@@ -2192,15 +2269,15 @@ TkActivateMenuEntry(menuPtr, index)
 	 * might already have been changed to disabled).
 	 */
 
-	if (mePtr->state == tkActiveUid) {
-	    mePtr->state = tkNormalUid;
+	if (mePtr->state == TK_STATE_ACTIVE) {
+	    mePtr->state = TK_STATE_NORMAL;
 	}
 	TkEventuallyRedrawMenu(menuPtr, menuPtr->entries[menuPtr->active]);
     }
     menuPtr->active = index;
     if (index >= 0) {
 	mePtr = menuPtr->entries[index];
-	mePtr->state = tkActiveUid;
+	mePtr->state = TK_STATE_ACTIVE;
 	TkEventuallyRedrawMenu(menuPtr, mePtr);
     }
     return result;
@@ -2266,6 +2343,7 @@ TkPostCommand(menuPtr)
  *--------------------------------------------------------------
  */
 
+
 static int
 CloneMenu(menuPtr, widget, newMenuTypeString)
     TkMenu *menuPtr;		/* The menu we are going to clone */
@@ -2307,7 +2385,7 @@ CloneMenu(menuPtr, widget, newMenuTypeString)
     	    Tcl_NewStringObj("MenuDup", -1));
     Tcl_ListObjAppendElement(menuPtr->interp, commandObjPtr,
     	    LangCopyArg(LangWidgetArg(menuPtr->interp, menuPtr->tkwin)));
-    Tcl_ListObjAppendElement(menuPtr->interp, commandObjPtr, *widget);
+    Tcl_ListObjAppendElement(menuPtr->interp, commandObjPtr, LangCopyArg(*widget));
     Tcl_ListObjAppendElement(menuPtr->interp, commandObjPtr,
     		Tcl_NewStringObj(newMenuTypeString, -1));
     Tcl_IncrRefCount(commandObjPtr);
@@ -2326,6 +2404,7 @@ CloneMenu(menuPtr, widget, newMenuTypeString)
     	TkMenu *newMenuPtr = menuRefPtr->menuPtr;
 	Arg newArgv[3];
 	int i, numElements;
+
 	*widget = newMenuName;
 
 	/*
@@ -2357,19 +2436,18 @@ CloneMenu(menuPtr, widget, newMenuTypeString)
    	if (Tk_BindtagsCmd((ClientData)newMenuPtr->tkwin, 
    		newMenuPtr->interp, 2, newArgv) == TCL_OK) {
    	    char *windowName;
-   	    Tcl_Obj *bindingsPtr = Tcl_GetObjResult(newMenuPtr->interp);
+   	    Tcl_Obj *bindingsPtr = LangScalarResult(newMenuPtr->interp);
    	    Tcl_Obj *elementPtr;
 
-            Tcl_IncrRefCount(bindingsPtr);
-
    	    Tcl_ListObjLength(newMenuPtr->interp, bindingsPtr, &numElements);
+
    	    for (i = 0; i < numElements; i++) {
    	    	Tcl_ListObjIndex(newMenuPtr->interp, bindingsPtr, i,
 			&elementPtr);
    	    	windowName = Tcl_GetStringFromObj(elementPtr, NULL);
    	    	if (strcmp(windowName, Tk_PathName(newMenuPtr->tkwin))
    	    		== 0) {
-   	    	    Tcl_Obj *newElementPtr = LangWidgetArg(menuPtr->interp, newMenuPtr->masterMenuPtr->tkwin); 
+   	    	    Tcl_Obj *newElementPtr = Tcl_NewStringObj(Tk_PathName(newMenuPtr->masterMenuPtr->tkwin),-1); 
    	    	    Tcl_ListObjReplace(menuPtr->interp, bindingsPtr,
    	    	    	    i + 1, 0, 1, &newElementPtr);
    	    	    newArgv[2] = bindingsPtr;
@@ -2379,7 +2457,7 @@ CloneMenu(menuPtr, widget, newMenuTypeString)
    	    	}
    	    }
    	    Tcl_DecrRefCount(bindingsPtr);   	    
-   	}
+   	}  
 	Tcl_DecrRefCount(newArgv[0]);
    	Tcl_ResetResult(menuPtr->interp);
       	
@@ -2409,7 +2487,7 @@ CloneMenu(menuPtr, widget, newMenuTypeString)
 		    newArgv[1] = newCascadeName;
 		    ConfigureMenuEntry(newMenuPtr->entries[i], 2, newArgv, 
 		    	    TK_CONFIG_ARGV_ONLY);
-		    Tcl_DecrRefCount(newArgv[0]);
+		    Tcl_DecrRefCount(newArgv[0]);   
    	    	}
    	    }
    	}
