@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkTextTag.c 1.36 96/03/07 15:30:43
+ * RCS: @(#) $Id: tkTextTag.c,v 1.2 1998/09/14 18:23:19 stanton Exp $
  */
 
 #include "default.h"
@@ -23,6 +23,28 @@
  * Information used for parsing tag configuration information:
  */
 
+static Tk_CustomOption stateOption = {
+    Tk_StateParseProc,
+    Tk_StatePrintProc, (ClientData) 2
+};
+
+/* The "-elide" is only provided for compatibility with TkMan 2.0,
+ * but might be removed in the future. The option "-state" provides
+ * the same functionality and is preferred.
+ */
+
+static int	ElideParseProc _ANSI_ARGS_((ClientData clientData,
+		    Tcl_Interp *interp, Tk_Window tkwin,
+		    Arg value, char *widgRec, int offset));
+static Arg	ElidePrintProc _ANSI_ARGS_((ClientData clientData,
+		    Tk_Window tkwin, char *widgRec, int offset,
+		    Tcl_FreeProc **freeProcPtr));
+
+static Tk_CustomOption elideOption = {
+    ElideParseProc,
+    ElidePrintProc, (ClientData) 0
+};
+
 static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_BORDER, "-background", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, border), TK_CONFIG_NULL_OK},
@@ -31,10 +53,13 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
     {TK_CONFIG_STRING, "-borderwidth", (char *) NULL, (char *) NULL,
 	"0", Tk_Offset(TkTextTag, bdString),
 		TK_CONFIG_DONT_SET_DEFAULT|TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-elide", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(TkTextTag, state),
+	TK_CONFIG_NULL_OK, &elideOption},
     {TK_CONFIG_BITMAP, "-fgstipple", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, fgStipple), TK_CONFIG_NULL_OK},
     {TK_CONFIG_FONT, "-font", (char *) NULL, (char *) NULL,
-	(char *) NULL, Tk_Offset(TkTextTag, fontPtr), TK_CONFIG_NULL_OK},
+	(char *) NULL, Tk_Offset(TkTextTag, tkfont), TK_CONFIG_NULL_OK},
     {TK_CONFIG_COLOR, "-foreground", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, fgColor), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-justify", (char *) NULL, (char *) NULL,
@@ -58,14 +83,17 @@ static Tk_ConfigSpec tagConfigSpecs[] = {
 	(char *) NULL, Tk_Offset(TkTextTag, spacing2String), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-spacing3", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, spacing3String), TK_CONFIG_NULL_OK},
+    {TK_CONFIG_CUSTOM, "-state", (char *) NULL, (char *) NULL,
+	(char *) NULL, Tk_Offset(TkTextTag, state),
+	TK_CONFIG_NULL_OK, &stateOption},
     {TK_CONFIG_LANGARG, "-tabs", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, tabString), TK_CONFIG_NULL_OK},
     {TK_CONFIG_LANGARG, "-underline", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, underlineString),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_UID, "-wrap", (char *) NULL, (char *) NULL,
+    {TK_CONFIG_CUSTOM, "-wrap", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, wrapMode),
-	TK_CONFIG_NULL_OK},
+	TK_CONFIG_NULL_OK, &textWrapModeOption},
     {TK_CONFIG_LANGARG, "-data", (char *) NULL, (char *) NULL,
 	(char *) NULL, Tk_Offset(TkTextTag, userData),
 	TK_CONFIG_NULL_OK},
@@ -223,24 +251,25 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    |Button2MotionMask|Button3MotionMask|Button4MotionMask
 		    |Button5MotionMask|ButtonPressMask|ButtonReleaseMask
 		    |EnterWindowMask|LeaveWindowMask|KeyPressMask
-		    |KeyReleaseMask|PointerMotionMask)) {
+		    |KeyReleaseMask|PointerMotionMask|VirtualEventMask)) {
 		Tk_DeleteBinding(interp, textPtr->bindingTable,
 			(ClientData) tagPtr, argv[4]);
 		Tcl_ResetResult(interp);
 		Tcl_AppendResult(interp, "requested illegal events; ",
-			"only key, button, motion, and enter/leave ",
+			"only key, button, motion, enter, leave, and virtual ",
 			"events may be used", (char *) NULL);
 		return TCL_ERROR;
 	    }
 	} else if (argc == 5) {
-	    LangCallback *command;
+	    Arg command;
     
 	    command = Tk_GetBinding(interp, textPtr->bindingTable,
 		    (ClientData) tagPtr, argv[4]);
 	    if (command == NULL) {
 		return TCL_ERROR;
 	    }
-	    Tcl_ArgResult(interp,LangCallbackArg(command));
+	    Tcl_ArgResult(interp,command);
+	    Tcl_DecrRefCount(command);
 	} else {
 	    Tk_GetAllBindings(interp, textPtr->bindingTable,
 		    (ClientData) tagPtr);
@@ -381,14 +410,11 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    return TCL_ERROR;
 		}
 	    }
-	    if ((tagPtr->wrapMode != NULL)
-		    && (tagPtr->wrapMode != tkTextCharUid)
-		    && (tagPtr->wrapMode != tkTextNoneUid)
-		    && (tagPtr->wrapMode != tkTextWordUid)) {
-		Tcl_AppendResult(interp, "bad wrap mode \"", tagPtr->wrapMode,
-			"\": must be char, none, or word", (char *) NULL);
-		tagPtr->wrapMode = NULL;
-		return TCL_ERROR;
+	    if (tagPtr->elideString != NULL) {
+		if (Tcl_GetBoolean(interp, tagPtr->elideString,
+			&tagPtr->elide) != TCL_OK) {
+		    return TCL_ERROR;
+		}
 	    }
 
 	    /*
@@ -409,7 +435,7 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    || (tagPtr->bdString != NULL)
 		    || (tagPtr->reliefString != NULL)
 		    || (tagPtr->bgStipple != None)
-		    || (tagPtr->fgColor != NULL) || (tagPtr->fontPtr != None)
+		    || (tagPtr->fgColor != NULL) || (tagPtr->tkfont != None)
 		    || (tagPtr->fgStipple != None)
 		    || (tagPtr->justifyString != NULL)
 		    || (tagPtr->lMargin1String != NULL)
@@ -422,7 +448,8 @@ TkTextTagCmd(textPtr, interp, argc, argv)
 		    || (tagPtr->spacing3String != NULL)
 		    || (tagPtr->tabString != NULL)
 		    || (tagPtr->underlineString != NULL)
-		    || (tagPtr->wrapMode != NULL)) {
+		    || (tagPtr->state != TK_STATE_NULL)
+		    || (tagPtr->wrapMode != TEXT_WRAPMODE_NULL)) {
 		tagPtr->affectsDisplay = 1;
 	    }
 	    TkTextRedrawTag(textPtr, (TkTextIndex *) NULL,
@@ -791,7 +818,7 @@ TkTextCreateTag(textPtr, tagName)
     tagPtr->relief = TK_RELIEF_FLAT;
     tagPtr->bgStipple = None;
     tagPtr->fgColor = NULL;
-    tagPtr->fontPtr = NULL;
+    tagPtr->tkfont = NULL;
     tagPtr->fgStipple = None;
     tagPtr->justifyString = NULL;
     tagPtr->justify = TK_JUSTIFY_LEFT;
@@ -815,7 +842,8 @@ TkTextCreateTag(textPtr, tagName)
     tagPtr->tabArrayPtr = NULL;
     tagPtr->underlineString = NULL;
     tagPtr->underline = 0;
-    tagPtr->wrapMode = NULL;
+    tagPtr->state = TK_STATE_NULL;
+    tagPtr->wrapMode = TEXT_WRAPMODE_NULL;
     tagPtr->userData = NULL;
     tagPtr->affectsDisplay = 0;
     textPtr->numTags++;
@@ -902,15 +930,16 @@ TkTextFreeTag(textPtr, tagPtr)
     if (tagPtr->fgColor != None) {
 	Tk_FreeColor(tagPtr->fgColor);
     }
-    if (tagPtr->fontPtr != None) {
-	Tk_FreeFontStruct(tagPtr->fontPtr);
-    }
+    Tk_FreeFont(tagPtr->tkfont);
     if (tagPtr->fgStipple != None) {
 	Tk_FreeBitmap(textPtr->display, tagPtr->fgStipple);
     }
     if (tagPtr->underlineString != NULL) {
 	LangFreeArg(tagPtr->underlineString,TCL_DYNAMIC);
     }
+    if (tagPtr->elideString != NULL) {
+	LangFreeArg(tagPtr->elideString,TCL_DYNAMIC);
+    }
     if (tagPtr->justifyString != NULL) {
 	ckfree(tagPtr->justifyString);
     }
@@ -1154,7 +1183,7 @@ TkTextBindProc(clientData, eventPtr)
 		mask = 0;
 		break;
 	}
-	if ((eventPtr->xbutton.state & AnyButtonMask) == mask) {
+	if ((eventPtr->xbutton.state & AnyButtonMask) == (unsigned) mask) {
 	    textPtr->flags &= ~BUTTON_DOWN;
 	    repick = 1;
 	}
@@ -1386,3 +1415,90 @@ TkTextPickCurrent(textPtr, eventPtr)
 	ckfree((char *) copyArrayPtr);
     }
 }
+
+/*
+ *--------------------------------------------------------------
+ *
+ * ElideParseProc --
+ *
+ *	This procedure is invoked during option processing to handle
+ *	the "-elide" option. This option is an obsolete equivalent
+ *	of the "-state" option. It is only added for compatibility
+ *	with TkMan 2.0, but will be removed in the future.
+ *
+ * Results:
+ *	A standard Tcl return value.
+ *
+ * Side effects:
+ *	The state for a given item gets replaced by the state
+ *	indicated in the value argument.
+ *
+ *--------------------------------------------------------------
+ */
+
+static int
+ElideParseProc(clientData, interp, tkwin, value, widgRec, offset)
+    ClientData clientData;		/* some flags.*/
+    Tcl_Interp *interp;			/* Used for reporting errors. */
+    Tk_Window tkwin;			/* Window containing canvas widget. */
+    Arg value;				/* Value of option. */
+    char *widgRec;			/* Pointer to record for item. */
+    int offset;				/* Offset into item. */
+{
+    int b;
+    register Tk_State *statePtr = (Tk_State *) (widgRec + offset);
+    char *svalue = LangString(value);
+
+    if(svalue == NULL || *svalue == 0) {
+	*statePtr = TK_STATE_NULL;
+	return TCL_OK;
+    }
+    if (Tcl_GetBoolean(interp, value, &b) != TCL_OK) {
+	*statePtr = TK_STATE_NULL;
+	return TCL_ERROR;
+    }
+    *statePtr = b?TK_STATE_HIDDEN:TK_STATE_NORMAL;
+    return TCL_OK;
+}
+/*
+ *--------------------------------------------------------------
+ *
+ * ElidePrintProc --
+ *
+ *	This procedure is invoked by the Tk configuration code
+ *	to produce a printable string for the "-elide"
+ *	configuration option.
+ *
+ * Results:
+ *	The return value is a string describing the state for
+ *	the item referred to by "widgRec".  In addition, *freeProcPtr
+ *	is filled in with the address of a procedure to call to free
+ *	the result string when it's no longer needed (or NULL to
+ *	indicate that the string doesn't need to be freed).
+ *
+ * Side effects:
+ *	None.
+ *
+ *--------------------------------------------------------------
+ */
+
+static Arg
+ElidePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
+    ClientData clientData;		/* Ignored. */
+    Tk_Window tkwin;			/* Window containing text widget. */
+    char *widgRec;			/* Pointer to record for item. */
+    int offset;				/* Offset into item. */
+    Tcl_FreeProc **freeProcPtr;		/* Pointer to variable to fill in with
+					 * information about how to reclaim
+					 * storage for return string. */
+{
+    register Tk_State *statePtr = (Tk_State *) (widgRec + offset);
+
+    if (*statePtr==TK_STATE_HIDDEN) {
+	return Tcl_NewIntObj(1);
+    } else if (*statePtr==TK_STATE_NULL) {
+	return LangStringArg("");
+    } else {
+	return Tcl_NewIntObj(0);
+    }
+}
