@@ -160,9 +160,13 @@ static Tk_ConfigSpec configSpecs[] = {
 };	
 
 #define DEF_TLISTENTRY_STATE	 "normal"
+#define DEF_TLISTENTRY_DATA	 NULL
 
 static Tk_ConfigSpec entryConfigSpecs[] = {
 
+    {TK_CONFIG_LANGARG, "-data", (char *) NULL, (char *) NULL,
+       DEF_TLISTENTRY_DATA, Tk_Offset(ListEntry, data), TK_CONFIG_NULL_OK},
+
     {TK_CONFIG_UID, "-state", (char*)NULL, (char*)NULL,
        DEF_TLISTENTRY_STATE, Tk_Offset(ListEntry, state), 0},
 
@@ -283,7 +287,7 @@ Tix_TListCmd(clientData, interp, argc, argv)
     int argc;			/* Number of arguments. */
     char **argv;		/* Argument strings. */
 {
-    Tk_Window main = (Tk_Window) clientData;
+    Tk_Window mainw = (Tk_Window) clientData;
     WidgetPtr wPtr;
     Tk_Window tkwin;
 
@@ -293,7 +297,7 @@ Tix_TListCmd(clientData, interp, argc, argv)
 	return TCL_ERROR;
     }
 
-    tkwin = Tk_CreateWindowFromPath(interp, main, argv[1], (char *) NULL);
+    tkwin = Tk_CreateWindowFromPath(interp, mainw, argv[1], (char *) NULL);
     if (tkwin == NULL) {
 	return TCL_ERROR;
     }
@@ -1300,11 +1304,7 @@ Tix_TLInfo(clientData, interp, argc, argv)
 	     chPtr=chPtr->next, i++) {
 	    
 	    if (chPtr->selected) {
-		if (i) {
-		    Tcl_AppendResult(interp, " ", (char *) NULL);
-		}
-		sprintf(buffer, "%d", i);
-		Tcl_AppendResult(interp, buffer, (char *) NULL);
+		Tcl_IntResults(interp,1,1,i);
 	    }
 	}
 	return TCL_OK;
@@ -2159,7 +2159,8 @@ Realloc(wPtr, new_size)
     if (new_size == wPtr->numRowAllocd) {
 	return;
     }
-    wPtr->rows = (ListRow*)ckrealloc((char *)(wPtr->rows), sizeof(ListRow)*new_size);
+    wPtr->rows = (ListRow*)ckrealloc( (char *)wPtr->rows,
+                                      sizeof(ListRow)*new_size);
     wPtr->numRowAllocd = new_size;
 }
 
@@ -2347,21 +2348,21 @@ RedrawRows(wPtr, pixmap)
 
 	for (r=0, n=0, chPtr=(ListEntry*)wPtr->entList.head; chPtr;
 		chPtr=chPtr->next, n++) {
-	    if (chPtr == wPtr->seeElemPtr) {
-		size = wPtr->rows[r].size[i];
-		break;
-	    }
 	    if (n == wPtr->rows[r].numEnt) {
 		n=0;
 		r++;
 		start += wPtr->rows[r].size[i];
 	    }
-	}
+	    if (chPtr == wPtr->seeElemPtr) {
+		size = wPtr->rows[r].size[i];
+		break;
+	    }
+	} 
 
-	if (wPtr->scrollInfo[i].offset + windowSize > start + size) {
+	if ((wPtr->scrollInfo[i].offset + windowSize) < (start + size)) {
 	    wPtr->scrollInfo[i].offset = start + size - windowSize;
 	}
-	if (wPtr->scrollInfo[i].offset < start) {
+	if (wPtr->scrollInfo[i].offset > start) {
 	    wPtr->scrollInfo[i].offset = start;
 	}
 	if (wPtr->scrollInfo[i].offset != old) {
@@ -2424,6 +2425,7 @@ RedrawRows(wPtr, pixmap)
 		flags |= TIX_DITEM_SELECTED_BG;
 	    }
 
+#if 0
 	    if (wPtr->isVertical) {
 		W = wPtr->rows[r].size[0];
 		H = chPtr->iPtr->base.size[1];
@@ -2431,6 +2433,10 @@ RedrawRows(wPtr, pixmap)
 		H = wPtr->rows[r].size[1];
 		W = chPtr->iPtr->base.size[0];
 	    }
+#else
+	    W = wPtr->maxSize[0];
+	    H = wPtr->maxSize[1];
+#endif
 
 	    Tix_DItemDisplay(pixmap, None, chPtr->iPtr, p[0], p[1], W, H,
 		flags);
